# -*- coding: utf-8 -*-

import logging

from numpy import *
import numpy.matlib
from numpy.matlib import repmat

import pyhrf

from pyhrf.ndarray import xndarray
from pyhrf.jde.samplerbase import *
from pyhrf.jde.nrl.bigaussian import NRLSampler
from pyhrf.jde.intensivecalc import sampleSmmNrl

logger = logging.getLogger(__name__)


class NRLwithHabSampler(NRLSampler):
    """
    Class handling the Gibbs sampling of Neural Response Levels in combination
    with habituation speed factor sampling. The underlying model is exponential decaying
    #TODO : comment attributes
    """

    P_TRUE_HABITS = 'trueHabits'
    P_SAMPLE_HABITS = 'sampleHabit'  # flag for habituation speed sampling
    # flag for definition of initial habituation speed value
    P_HABITS_INI = 'habitIni'
    # lambda-like parameter of the Laplacian distribution for function
    # habitSample()
    P_HAB_ALGO_PARAM = 'paramLexp'
    P_OUTPUT_RATIO = 'outputRatio'

    # defaultParameters.update( {
    #                 #parameters for habituation sampling
    #                 P_HABITS_INI : None,
    #                 P_SAMPLE_HABITS : 1,
    #                 P_HAB_ALGO_PARAM : 5.,
    #                 P_OUTPUT_RATIO : 0,
    #             })

    parametersComments = NRLSampler.parametersComments.copy()
    parametersComments[
        P_HAB_ALGO_PARAM] = 'lambda-like parameter of the Laplacian distribution in habit sampling\n recommended between 1. and 10.'

    def __init__(self):

        NRLSampler.__init__(self)

# Comment setting of beta parameter since it is properly handled in the base class
#        self.beta = self.parameters[self.P_BETA]

        # habituation speed sampling parameters
        self.sampleHabitFlag = self.parameters[self.P_SAMPLE_HABITS]
        # Habituation initialization
        self.habits = self.parameters[self.P_HABITS_INI]
        if self.parameters.has_key(self.P_TRUE_HABITS):
            # load true habits if exist
            self.trueHabits = params[self.P_TRUE_HABITS]
        else:
            self.trueHabits = None
        # parametre pour la Laplacienne tronquee
        self.Lexp = self.parameters[self.P_HAB_ALGO_PARAM]

        # pour la sauvegarde des donnees
        self.habitsHistory = None
        self.outputRatio = self.parameters[
            self.P_OUTPUT_RATIO]  # affiche ou pas les ratio
        if self.outputRatio:
            self.ratioHistory = None  # pour voir les ratio
            self.ratiocourbeHistory = None  # pour voir la courbe des ratio

        self.labelsColors = self.parameters[self.P_LABELS_COLORS]

    def linkToData(self, dataInput):

        NRLSampler.linkToData(self, dataInput)

        # recuperation de X (matrice (nbCond * ny * nh) )
        self.varSingleCondXtrials = self.dataInput.varSingleCondXtrials

        # pour les onsets
        #self.nbTrials=zeros((self.nbSessions,self.nbConditions), dtype=int)
        # for isess in xrange(self.nbSessions):

        self.onsets = self.dataInput.onsets
        self.lastonset = 0.
        self.deltaOns = {}
        self.nbTrials = zeros(self.nbConditions, dtype=int)
        for nc in xrange(self.nbConditions):
            self.deltaOns[nc] = numpy.diff(self.onsets[nc])
            self.nbTrials[nc] = len(self.onsets[nc])
            self.lastonset = max(
                self.lastonset, self.onsets[nc][self.nbTrials[nc] - 1])

        # astuce pour les donnees reelles -> pour remettre 'a zero' tout les 4
        # occurences
            for trial in xrange(self.nbTrials[nc] - 1):
                if (((trial + 1) % 4) == 0):
                    self.deltaOns[nc][trial] = 100.
                else:
                    # on divise les deltaOns par 4. pour prendre plus en compte
                    # les valeurs anterieures d'habituation
                    self.deltaOns[nc][trial] = self.deltaOns[nc][trial] / 4.

        # calcul du dernier onset -> pour afficher les timesNRLs
        self.lastonset = int(self.lastonset + 1)
        # print self.lastonset

        # determination du nbGammas utile dans spExtract -> mais il y a une
        # erreur pour Xmask (dans sparsedot
        self.nbGammas = range(self.nbConditions)
        self.nnulls = range(self.nbConditions)
        self.Xmask = range(self.nbConditions)
        for nc in xrange(self.nbConditions):
            self.Xmask[nc] = range(self.nbTrials[nc])
            self.nbGammas[nc] = zeros(self.nbTrials[nc], dtype=int)
            self.nnulls[nc] = zeros(
                (self.varSingleCondXtrials[nc, :, :] == 0).sum())
            for i in xrange(self.nbTrials[nc]):
                #self.nbGammas[j][i] = (self.varSingleCondXtrials[j,:,:] == (i + 1)).sum()
                self.Xmask[nc][i] = transpose(
                    where(self.varSingleCondXtrials[nc, :, :] == (i + 1)))
                self.nbGammas[nc][i] = shape(self.Xmask[nc][i])[0]

        logger.info('deltaOns :')
        logger.info(self.deltaOns)

    def checkAndSetInitValue(self, variables):

        # init NRL and Labels
        NRLSampler.checkAndSetInitValue(self, variables)
        self.checkAndSetInitHabit(variables)

    def checkAndSetInitHabit(self, variables):

        # init habituation speed factors and time-varying NRLs
        if self.habits == None:  # if no habituation speed specified
            if not self.sampleHabitFlag:
                # Attention: on a un probleme si on fait tourner ce modele sur
                # des donnees simulees par le modele stationnaire. Dans ce cas,
                # il faut forcer ici a passer et prendre des habits nulles
                if self.dataInput.simulData != None:
                    # using simulated Data for HABITUATION sampling
                    print "load Habituation from simulData", self.dataInput.simulData.habitspeed.data
                    self.habits = self.dataInput.simulData.habitspeed.data
                    self.timeNrls = self.dataInput.simulData.nrls.timeNrls
                    self.Gamma = self.setupGamma()
                else:  # sinon, on prend que des zeros (modele stationnaire)
                    self.habits = numpy.zeros(
                        (self.nbConditions, self.nbVox), dtype=float)
                    self.setupTimeNrls()
            else:
                logger.info("Uniform set up of habituation factors")
                habitCurrent = numpy.zeros(
                    (self.nbConditions, self.nbVox), dtype=float)
                for nc in xrange(self.nbConditions):
                    habitCurrent[nc, self.voxIdx[1][nc]] = numpy.random.rand(
                        self.cardClass[1][nc])
                self.habits = habitCurrent

        if self.outputRatio:
            self.ratio = zeros((self.nbConditions, self.nbVox, 2), dtype=float)
            self.ratiocourbe = zeros(
                (self.nbConditions, self.nbVox, 100, 5), dtype=float)
            self.compteur = numpy.zeros(
                (self.nbConditions, self.nbVox), dtype=float)

        self.setupTimeNrls()
        logger.info('habituation initiale')
        logger.debug(self.habits)

    def initObservables(self):
        NRLSampler.initObservables(self)
        # sauvegarde des habits et nrls
        self.nrlsHistory = None
        self.meanHabits = None
        # Toutes les habituations par voxels
        self.cumulHabits = zeros((self.nbConditions, self.nbVox), dtype=float)
        self.meanHabitsCond = zeros(
            (self.nbClasses, self.nbConditions, self.nbVox), dtype=float)
        self.cumulHabitsCond = zeros(
            (self.nbClasses, self.nbConditions, self.nbVox), dtype=float)
        self.voxelActivity = zeros(
            (self.nbClasses, self.nbConditions, self.nbVox), dtype=int)
        self.obs = 0

    def updateObsersables(self):
        NRLSampler.updateObsersables(self)
        self.obs += 1
        self.cumulHabits += self.habits
       #self.meanHabits = self.cumulHabits / self.nbItObservables
        self.meanHabits = self.cumulHabits / self.obs
        temp = zeros((self.nbConditions, self.nbVox), dtype=float)
        for c in xrange(self.nbClasses):
            putmask(temp, self.labels == c, self.habits)
            self.cumulHabitsCond[c, :, :] += temp
            self.voxelActivity[c, :, :] += self.labels == c
        self.meanHabitsCond = self.cumulHabitsCond / self.voxelActivity

    def cleanObservables(self):
        NRLSampler.cleanObservables(self)
        del self.cumulHabits
    # self.cleanMemory()

    def saveCurrentValue(self):

        NRLSampler.saveCurrentValue(self)

        if self.keepSamples:
            if (self.iteration % self.sampleHistoryPace) == 0:
                # if self.habitsHistory != None:
                    #self.habitsHistory = concatenate((self.habitsHistory,[self.habits]))
                # else:
                    #self.habitsHistory = [self.habits]
                if self.habitsHistory != None:
                    self.habitsHistory = concatenate(
                        (self.habitsHistory, [self.habits]))
                else:
                    self.habitsHistory = array([self.habits])
                if self.nrlsHistory != None:
                    self.nrlsHistory = concatenate(
                        (self.nrlsHistory, [self.currentValue]))
                else:
                    self.nrlsHistory = array([self.currentValue])
        if self.outputRatio:
            if self.ratioHistory != None:
                self.ratioHistory = concatenate(
                    (self.ratioHistory, [self.ratio]))
            else:
                self.ratioHistory = array([self.ratio])
            if self.ratiocourbeHistory != None:
                self.ratiocourbeHistory = concatenate(
                    (self.ratiocourbeHistory, [self.ratiocourbe]))
            else:
                self.ratiocourbeHistory = array([self.ratiocourbe])

    def samplingWarmUp(self, variables):

        logger.info('Habit Sampling Warm up')

        # issu de NRLSampler
        #---------------------------------------------------------------

        smplHRF = self.get_variable('hrf')
        varHRF = smplHRF.currentValue
        self.nh = len(varHRF)            # taille de la HRF

        self.varXh = zeros(
            (self.nbVox, self.ny, self.nbConditions), dtype=float)
        self.aXh = zeros((self.ny, self.nbVox, self.nbConditions), dtype=float)
        #self.vycArray = zeros((self.nbVox, self.ny, self.nbConditions))
        self.sumaX = zeros((self.ny, self.nh, self.nbVox), dtype=float)

        #self.sumaXQ = zeros((self.ny,self.nh , self.nbVox), dtype = float)

        self.updateXh(varHRF)

        self.varYtilde = zeros((self.ny, self.nbVox), dtype=float)
        self.sumaXh = zeros((self.ny, self.nbVox), dtype=float)
        self.updateYtilde()

        self.varXhtQ = zeros(
            (self.nbVox, self.nbConditions, self.ny), dtype=float)

        # utilise par noise.py
        #self.sumaXtQ = zeros((self.nbVox, self.nh, self.ny) , dtype=float)
        #self.sumaXtQaX = zeros((self.nbVox, self.nh, self.nh), dtype=float)
        self.sumaXhtQaXh = zeros(self.nbVox, dtype=float)

        self.varClassApost = zeros((self.nbClasses, self.nbConditions, self.nbVox),
                                   dtype=float)
        self.sigClassApost = zeros((self.nbClasses, self.nbConditions, self.nbVox),
                                   dtype=float)
        self.meanClassApost = zeros((self.nbClasses, self.nbConditions, self.nbVox),
                                    dtype=float)
        self.meanApost = zeros((self.nbConditions, self.nbVox), dtype=float)
        self.sigApost = zeros((self.nbConditions, self.nbVox), dtype=float)

        self.interMeanClassA = zeros(
            (self.nbConditions, self.nbVox), dtype=float)

        self.corrEnergies = zeros((self.nbConditions, self.nbVox), dtype=float)

        self.aa = zeros((self.nbConditions, self.nbConditions, self.nbVox),
                        dtype=float)
        self.computeAA(self.currentValue, self.aa)

        self.iteration = 0

    def updateXh(self, varHRF):
        temp = empty((self.nbVox, self.nbConditions), dtype=float)
        self.sumaX = zeros((self.ny, self.nh, self.nbVox), dtype=float)
        #self.sumaXQ = zeros((self.ny,self.nh , self.nbVox), dtype = float)
        for nv in xrange(self.nbVox):
            for nc in xrange(self.nbConditions):
                self.updateGammaTimeNRLs(nc, nv)
                Xtilde = self.spExtract(
                    self.Gamma[nc][nv, :], self.varSingleCondXtrials[nc, :, :], nc)
                aXtilde = self.spExtract(
                    self.timeNrls[nc][nv, :], self.varSingleCondXtrials[nc, :, :], nc)
                self.sumaX[:, :, nv] += Xtilde
                self.varXh[nv, :, nc] = dot(Xtilde, varHRF)

                # ici, j ai fait un essai avec sparsedot, mais ca ne marche pas...
                #Xhtemp = zeros(self.ny, dtype = float)
                #aXhtemp = zeros(self.ny, dtype = float)
                # for t in xrange(self.nbTrials[nc]):
                #Xhtemp += sparsedotdimun(self.Gamma[nc][nv,t], varHRF, self.Xmask[nc][t], self.ny)
                #aXhtemp += sparsedotdimun(self.timeNrls[nc][nv,t], varHRF, self.Xmask[nc][t], self.ny)
                ##self.sumaXQ[:,:,nv] += sparsedot(self.timeNrls[nc][nv,t], self.dataInput.delta, self.Xmask[nc][t], (self.ny, self.nh))
                #self.varXh[nv,:,nc]= Xhtemp

                self.aXh[:, nv, nc] = dot(aXtilde, varHRF)
                #self.aXh[:,nv,nc] = aXhtemp

                temp[nv, nc] = dot(
                    dot(self.aXh[:, nv, nc], self.dataInput.delta), self.aXh[:, nv, nc])

                #aXQaX = zeros((self.nh, self.nh), dtype=float)
                ##aXQaX = self.bilinearsparsedot(self.timeNrls[nc][nv,:], self.dataInput.delta, self.nh, nc)
                #temp[nv,nc] = dot(dot(varHRF, aXQaX), varHRF)

        # utile pour noise.py
        self.sumaXh = sum(self.aXh, axis=2)
        self.sumaXhtQaXh = sum(temp, axis=1)

    # doit etre precede d un updateXh pour avoir sumaXh
    def updateYtilde(self):
        self.varYtilde = self.dataInput.varMBY - self.sumaXh

    # initialisation of temporal-varying  NRLs (tout a 0)
    def setupTimeNrls(self):
        self.timeNrls = range(self.nbConditions)
        self.Gamma = range(self.nbConditions)
        for nc in xrange(self.nbConditions):
            self.timeNrls[nc] = numpy.zeros(
                (self.nbVox, self.nbTrials[nc]), dtype=float)
            self.Gamma[nc] = numpy.zeros(
                (self.nbVox, self.nbTrials[nc]), dtype=float)
            for nv in xrange(self.nbVox):
                self.updateGammaTimeNRLs(nc, nv)

    def updateGammaTimeNRLs(self, nc, nv):

        self.Gamma[nc][nv, 0] = 1.
        self.timeNrls[nc][nv, 0] = self.currentValue[nc, nv]
        skk = 0.
        for k in xrange(1, self.nbTrials[nc]):
            skk = (self.habits[nc, nv] ** self.deltaOns[nc]
                   [k - 1]) * (skk + self.timeNrls[nc][nv, k - 1])
            self.Gamma[nc][nv, k] = 1. / (1. + skk)
            self.timeNrls[nc][nv, k] = self.Gamma[
                nc][nv, k] * self.currentValue[nc, nv]

    def setupGamma(self):
        # lorsqu on connait les timeNRLs
        Gamma = range(self.nbConditions)
        for nc in xrange(self.nbConditions):
            Gamma[nc] = numpy.zeros(
                (self.nbVox, self.nbTrials[nc]), dtype=float)
            for k in xrange(self.nbTrials[nc]):
                Gamma[nc][:, k] = self.timeNrls[nc][
                    :, k] / self.currentValue[nc, :]
        return Gamma

    def computeVarXhtQ(self, Q):
        for nc in xrange(self.nbConditions):
            self.varXhtQ[:, nc, :] = dot(
                self.varXh[:, :, nc].swapaxes(0, 1).transpose(), Q)
            #self.varXhtQ[:,nc,:] = dot(self.varXh[:,:,nc], Q)
            #self.varXhtQXh[:, nc] = diag(dot(self.varXhtQ[:,nc,:],self.varXh[:,:,nc].swapaxes(0,1)))

    def computeComponentsApost(self, variables, j, XhtQXh):
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        var = sIMixtP.getCurrentVars()
        mean = sIMixtP.getCurrentMeans()
        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        nrls = self.currentValue

        # for j in xrange(self.nbConditions):
        gTQgjrb = XhtQXh[:, j] / rb   # de taille nbVox

        ej = self.varYtilde + \
            repmat(nrls[j, :], self.ny, 1) * self.varXh[:, :, j].swapaxes(0, 1)
        numpy.divide(
            diag(dot(self.varXhtQ[:, j, :], ej)), rb, self.varXjhtQjeji)

        # ici classe: 0 (inactif) ou 1 (actif)
        for c in xrange(self.nbClasses):
            self.varClassApost[c, j, :] = 1. / (1. / var[c, j] + gTQgjrb)
            numpy.sqrt(
                self.varClassApost[c, j, :], self.sigClassApost[c, j, :])
            if c > 0:  # assume 0 stands for inactivating class
                numpy.multiply(self.varClassApost[c, j, :],
                               add(mean[c, j] / var[c, j], self.varXjhtQjeji),
                               self.meanClassApost[c, j, :])
            else:
                multiply(self.varClassApost[c, j, :], self.varXjhtQjeji,
                         self.meanClassApost[c, j, :])

            logger.debug('meanClassApost %d cond %d :', c, j)
            logger.debug(self.meanClassApost[c, j, :])

    def habitCondSampler(self, j, rb, varHRF):

        # Samples habituation factors
        for i in xrange(self.nbVox):  # loops over voxels
            # compute new voxel-dependant design matrix
            # for j in xrange(self.nbConditions):## loops over conditions
            logger.debug('Condition %i :', j)
            if (self.labels[j, i] == 1):  # sampling the habituation factors
                eji = self.varYtilde[:, i] + self.aXh[:, i, j]

            # Generate a candidate for the habitutation parameter in voxel i
            newHabit, KhabNew = LaplacianPdf(
                self.Lexp, self.habits[j, i], 0., 1., 1)

            # pour afficher les courbes de ratio
            if self.keepSamples and self.outputRatio:
                if (self.iteration % self.sampleHistoryPace) == 0:
                    Dif2 = eji - self.aXh[:, i, j]
                    # on calcule les ratio tous les 1/100
                    for iii in range(100):
                        ii = iii / 100.
                        tirage = 0.
                        vrai = 0.
                        Gamma_ii, timeNrls_ii = subcptGamma(
                            self.currentValue[j, i], ii, self.nbTrials[j], self.deltaOns[j][:])
                        XsumWT_ii = self.spExtract(
                            timeNrls_ii, self.varSingleCondXtrials[j], j)
                        varG_ii = numpy.dot(XsumWT_ii, varHRF)
                        XsumWTb_ii = self.spExtract(
                            Gamma_ii, self.varSingleCondXtrials[j], j)
                        varGb_ii = numpy.dot(XsumWTb_ii, varHRF)

                        Dif1 = eji - varG_ii

                        Dif12QDif12 = dot(dot(Dif1.transpose(), self.dataInput.delta), Dif1) - dot(
                            dot(Dif2.transpose(), self.dataInput.delta), Dif2)
                        #Dif12QDif12=dot(Dif1.transpose(),Dif1) - dot(Dif2.transpose(),Dif2)

                        # probleme observe lorsque le ratio est trop grand
                        if Dif12QDif12 / rb[i] > -1400.:
                            Ratio1 = math.exp(-.5 * Dif12QDif12 / rb[i])
                        else:
                            Ratio1 = 1000000.
                        Ratio2 = (2. - math.exp(-self.Lexp * self.habits[j, i]) - math.exp(self.Lexp * (
                            self.habits[j, i] - 1.))) / (2. - math.exp(-self.Lexp * ii) - math.exp(self.Lexp * (ii - 1.)))
                        if abs(ii - newHabit) < 0.005:
                            tirage = min(Ratio1 * Ratio2, 1)

                        if abs(ii - self.trueHabits[j, i]) < 0.005:
                            vrai = 1.
                        if abs(ii - self.habits[j, i]) < 0.005:
                            vrai = 0.7

                        self.ratiocourbe[j, i, iii, :] = [
                            min(Ratio1, 10.), Ratio2, min(Ratio1 * Ratio2, 1), tirage, vrai]
            # calcul gamma - habituation factor
            GammaNew, timeNrlsNew = subcptGamma(
                self.currentValue[j, i], newHabit, self.nbTrials[j], self.deltaOns[j][:])
            # compute key quantities for the new candidate
            XsumWT_x = self.spExtract(
                timeNrlsNew, self.varSingleCondXtrials[j], j)
            varG_x = numpy.dot(XsumWT_x, varHRF)
            XsumWTb_x = self.spExtract(
                GammaNew, self.varSingleCondXtrials[j], j)
            varGb_x = numpy.dot(XsumWTb_x, varHRF)

            Dif1 = eji - varG_x
            Dif2 = eji - self.aXh[:, i, j]
#           Dif2 = self.varYtilde[:,i]

            #Dif12QDif12=dot(Dif1.transpose(),Dif1) - dot(Dif2.transpose(),Dif2)
            Dif12QDif12 = dot(dot(Dif1.transpose(), self.dataInput.delta),
                              Dif1) - dot(dot(Dif2.transpose(), self.dataInput.delta), Dif2)

            # probleme observe lorsque le ratio est trop grand -> on tronque
            # dans ce cas
            if Dif12QDif12 / rb[i] > -1400.:
                Ratio1 = math.exp(-.5 * Dif12QDif12 / rb[i])
            else:
                Ratio1 = 1000000.  # assure que Alpha = 1

            Ratio2 = (2. - math.exp(-self.Lexp * self.habits[j, i]) - math.exp(self.Lexp * (self.habits[j, i] - 1.))) / (
                2. - math.exp(-self.Lexp * newHabit) - math.exp(self.Lexp * (newHabit - 1.)))

            # TODO pour gagner du temps -> prendre KhabNew issu de LaplacianPdf
            # Ratio2 = self.Khab[j,i] / KhabNew   # probleme car KhabNew est en
            # fait l'ancienne valeur

            # sauvegarde du ratio
            if self.outputRatio:
                self.ratio[j, i, :] = [Ratio1, Ratio2]

            # Compute the acceptation rate of the MH algo
            Alpha = min(Ratio1 * Ratio2, 1)
            u = numpy.random.rand()
            if (u <= Alpha):
                self.habits[j, i] = newHabit

                # sauvegarde le nombre de fois que l'habituation a change
                if self.keepSamples and self.outputRatio:
                    self.compteur[j, i] += 1

            self.updateGammaTimeNRLs(j, i)

        else:  # Inactiv voxel
            self.habits[j, i] = 0.
            self.updateGammaTimeNRLs(j, i)

    def sampleNrlsSerial_bak(self, rb, h, varLambda, varCI, varCA,
                             meanCA, varXhtQXh, variables):

        for j in xrange(self.nbConditions):
            for i in random.permutation(self.nbVox):
                if self.iteration > 200:
                    logger.info('Doing vox %d, cond %d', i, j)
                    logger.info('---------------------')
                self.computeComponentsApostInVox(variables, j, i, gTQg)
                if self.sampleLabelsFlag:
                    self.sampleLabel(j, i, variables)
                label = self.labels[j, i]
                oldVal = self.currentValue[j, i]

    def sampleNrlsSerial(self, varXh, rb, h, varCI,
                         varCA, meanCA, variables):

        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        var = sIMixtP.getCurrentVars()
        mean = sIMixtP.getCurrentMeans()
        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        #varXh = self.get_variable('hrf').varXh
        nrls = self.currentValue
        #gTQgjrb = gTQg[j]/rb[i]
        neighbours = self.dataInput.neighboursIndexes

        for j in xrange(self.nbConditions):
            # for j in random.permutation(self.nbConditions):
            # print 'self.nbVox : ', self.nbVox
            voxOrder = random.permutation(self.nbVox)
# print 'voxOrder :', voxOrder
# print 'varXh : ', varXh.shape, varXh.dtype
# print 'neighbours :', neighbours
# print 'Components :'
# print 'mCA =', mean[self.L_CA],'vCA =', var[self.L_CA]
# print 'mCI =', mean[self.L_CI],'vCI =', var[self.L_CI]
# print 'beta = ',
            # print 'labelsSamples, cond %d -> mean=%f'
            # %(j,self.labelsSamples[j,:].mean())
            betaj = self.samplerEngine.get_variable('beta').currentValue[j]
            # print 'self.varYtilde:', self.varYtilde.shape
            sampleSmmNrl(voxOrder, rb, neighbours, self.varYtilde,
                         self.labels[j, :], varXh[:, :, j],
                         self.currentValue[j, :],
                         self.nrlsSamples[j, :], self.labelsSamples[j, :],
                         self.varXhtQ[:, j, :], array([0.]), betaj,
                         mean[:, j], var[:, j], self.nbClasses,
                         self.sampleLabelsFlag + 0, self.iteration, j)
            # print 'self.currentValue[j,:] :'
            # print self.currentValue[j,:]
            # print 'self.labels[j,:]:'
            # print self.labels[j,:]
            # sys.exit(1)
            self.countLabels(self.labels, self.voxIdx, self.cardClass)

    def sampleNrlsParallel(self, rb, h, varLambda, varCI, varCA,
                           meanCA, varXhtQXh, variables):

        pass

    def habitCondSamplerParallel(self, rb, h):
        pass

    def habitCondSamplerSerial(self, rb, h):
        pass

    def computeVarYTildeHab(self, varXh):
        # print 'varXh.shape:', varXh.shape
        # print 'self.varYtilde:', self.varYtilde.shape
        for i in xrange(self.nbVox):
            repNRLi = repmat(self.currentValue[:, i], self.ny, 1)
            aijXjh = repNRLi * varXh[i, :, :]
            # print 'aijXjh:', aijXjh.shape
            self.varYtilde[:, i] = self.dataInput.varMBY[
                :, i] - aijXjh.sum(axis=1)

    def computeVarYTildeHabOld(self, varXh):
        # yTilde_j = y_j - sum_m(a_j^m X^m h)
        logger.debug('computeVarYTildeOpt...')
        logger.debug('varXh:' + str(varXh.shape))
        self.aXh = repmat(varXh, self.nbVox, 1).reshape(self.nbVox,
                                                        self.ny,
                                                        self.nbConditions)

        self.aXh = self.aXh.swapaxes(0, 1).swapaxes(1, 2)
        self.aXh *= self.currentValue
        self.aXh.sum(1, out=self.sumaXh)
        logger.debug('sumaXh %s', str(self.sumaXh.shape))
        logger.debug(self.sumaXh)

        numpy.subtract(self.dataInput.varMBY, self.sumaXh, self.varYtilde)

        logger.debug('varYtilde %s', str(self.varYtilde.shape))
        logger.debug(self.varYtilde)

    def sampleNextInternal(self, variables):
        # TODO : comment

        # load mixture params et mixture weight
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        varCI = sIMixtP.currentValue[sIMixtP.I_VAR_CI]
        varCA = sIMixtP.currentValue[sIMixtP.I_VAR_CA]
        meanCA = sIMixtP.currentValue[sIMixtP.I_MEAN_CA]
        varLambda = variables[
            self.samplerEngine.I_WEIGHTING_PROBA].currentValue
        # load noise
        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        # load hrf
        sHrf = self.get_variable('hrf')
        #varXh = sHrf.varXh
        h = sHrf.currentValue
        #varHRF = self.get_variable('hrf').currentValue
        logger.info('iteration %i', self.iteration)

        self.updateXh(h)  # -> update self.varXh
        self.updateYtilde()
        self.computeVarXhtQ(self.dataInput.delta)

        self.labelsSamples = random.rand(self.nbConditions, self.nbVox)
        self.nrlsSamples = random.randn(self.nbConditions, self.nbVox)

        varXhtQXh = zeros((self.nbVox, self.nbConditions), dtype=float)
        for i in xrange(self.nbVox):
            varXhtQXh[i, :] = diag(
                dot(self.varXhtQ[i, :, :], self.varXh[i, :, :]))

        # Calcul des variables a post regroupe dans self.computeComponentsApost
        #self.computeLambdaAPost(varCI, varCA, varLambda)

        if self.samplerEngine.get_variable('beta').currentValue[0] <= 0:
            self.sampleNrlsParallel(rb, h, varLambda, varCI,
                                    varCA, meanCA, varXhtQXh, variables)
            self.computeVarYTildeHab(self.varXh)
            if self.sampleHabitFlag:
                logger.info(
                    '(it %i) Habituation conditional sampling parallel mode ...')
                self.habitCondSamplerParallel(rb, h)
                self.computeVarYTildeHab(self.varXh)
        else:
            self.sampleNrlsSerial(self.varXh, rb, h, varCI,
                                  varCA, meanCA, variables)
            self.computeVarYTildeHab(self.varXh)
            if self.sampleHabitFlag:
                logger.info(
                    '(it %i) Habituation conditional sampling serial mode ...')
                self.habitCondSamplerSerial(rb, h)
            self.saveSamples()
        self.updateXh(h)
        self.updateYtilde()
        self.computeAA(self.currentValue, self.aa)

        logger.info('NRLS current Value: ')
        logger.info(self.currentValue)

        for j in xrange(self.nbConditions):
            logger.info('All nrl cond %d:', j)
            logger.info(self.currentValue[j, :])
            logger.info('nrl cond %d = %1.3f(%1.3f)', j,
                        self.currentValue[j, :].mean(),
                        self.currentValue[j, :].std())
            for c in xrange(self.nbClasses):
                logger.info('All nrl %s cond %d:', self.CLASS_NAMES[c], j)
                ivc = self.voxIdx[c][j]
                logger.info(self.currentValue[j, ivc])

                logger.info('nrl %s cond %d = %1.3f(%1.3f)', self.CLASS_NAMES[c],
                            j, self.currentValue[j, ivc].mean(),
                            self.currentValue[j, ivc].std())

        self.iteration += 1  # TODO : factorize !!

    def sampleNextAlt(self, variables):  # if we don't want to sample Nrls

        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        varHRF = self.get_variable('hrf').currentValue

        logger.info('iteration %i', self.iteration)

        self.updateXh(varHRF)
        self.updateYtilde()

        # sampling labels
        if self.sampleLabelsFlag:
            self.computeVarXhtQ(self.dataInput.delta)
            self.labelsSamples = random.rand(self.nbConditions, self.nbVox)
            varXhtQXh = zeros((self.nbVox, self.nbConditions), dtype=float)
            for i in xrange(self.nbVox):
                varXhtQXh[i, :] = diag(
                    dot(self.varXhtQ[i, :, :], self.varXh[i, :, :]))

            for j in random.permutation(self.nbConditions):
                self.computeComponentsApost(variables, j, varXhtQXh)
                logger.info('(It %i) Sampling labels - cond %d ...',
                            self.iteration, j)
                self.sampleLabels(j, variables)
                logger.info('(It %i) Sampling labels done!', self.iteration)
                self.countLabels(self.labels, self.voxIdx, self.cardClass)

        # sampling habits
        if self.sampleHabitFlag:
            logger.info('(it %i) Habituation conditional sampling ...',
                        self.iteration)
            for j in random.permutation(self.nbConditions):
                self.habitCondSampler(j, rb, varHRF)
            logger.info('(it %i) update done ...', self.iteration)

        self.saveSamples()
        self.computeAA(self.currentValue, self.aa)
        self.iteration += 1

    def finalizeSampling(self):

        self.finalLabels = self.getFinalLabels()

        # on n'afiche que les habituations du label final
        for c in xrange(self.nbClasses):
            putmask(self.meanHabits, self.finalLabels ==
                    c, self.meanHabitsCond[c, :, :])

        # affichage de timeNRLs -> 10 voxels tires au hasard (car on ne peux
        # tout sauvegarder -> trop grand (nc * nVox * lastonset)
        self.outputTimeNRLs = zeros(
            (self.nbConditions, 10, self.lastonset), dtype=float)
        for nc in xrange(self.nbConditions):
            # tirage des voxels observes
            a = [int(self.nbVox * random.rand()) for i in range(10)]
            # si on veut choisir precisement les voxels, il faut passer par ipython (comme plus loin) avec getIndex([axial Coronal Sagittal])
            # il faut prendre exactement 10 voxels ou alors changer la def de self.outputTimeNRLs et le axesDomain de getOutput
            # attention, on observe les memes voxels pour toutes les regions, il faut donc il ne faut pas depasser le nbVox de la plus petite region
            # par ex:
            # a = [ min( 23, self.nbVox), min(36, self.nbVox), etc...]

            # important: print des voxels observes
            # print 'voxels observes', a
            for nt in xrange(self.nbTrials[nc]):
                # on sauvegarde les timeNRLs qui nous interessent
                self.outputTimeNRLs[
                    nc, :, int(self.onsets[nc][nt])] = self.timeNrls[nc][a, nt]
        self.voxelobs = a
        # sauvegarde dans result des voxels observes
        # pour trouver les coordonnes correspondant, il faut sous ipython:
        # charger les donnees: import cPickle
        #                      s = cPickle.load(open('result.pyd'))
        # pour voir les coor.: s[numeroROI]['sampler'].dataInput.voxelMapping.getCoord(voxelobs)
        #

        smplHRF = self.samplerEngine.get_variable('hrf')

        # Correct hrf*nrl scale ambiguity :
        scaleF = smplHRF.getScaleFactor()

        # Use HRF amplitude :
        logger.debug('scaleF=%1.2g', scaleF)
        logger.debug('self.finalValue : %1.2g - %1.2g', self.finalValue.min(),
                     self.finalValue.max())
        self.finalValueScaleCorr = self.finalValue * scaleF

        if self.computeContrastsFlag:
            self.computeContrasts()

    def getOutputs(self):

        outputs = NRLSampler.getOutputs(self)

        axes_names = ['condition', 'voxel']
        axes_domains = {'condition': self.dataInput.cNames}
        outputs['pm_Habits'] = xndarray(self.meanHabits,
                                        axes_names=axes_names,
                                        axes_domains=axes_domains,
                                        value_label="Habituation")

        axes_names = ['condition', 'numeroVox', 'time']
        axes_domains = {'condition': self.dataInput.cNames,
                        'numeroVox': [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
                        'time': range(self.lastonset)}
        outputs['pm_timeNrls'] = xndarray(self.outputTimeNRLs,
                                          axes_names=axes_names,
                                          axes_domains=axes_domains,
                                          value_label="TimeNRLs")

        dt = self.samplerEngine.get_variable('hrf').dt
        rpar = self.dataInput.paradigm.get_joined_and_rastered(dt)
        parmask = [where(rpar[c] == 1) for c in self.dataInput.cNames]
        tnrl = zeros((self.nbVox, self.nbConditions, len(rpar[rpar.keys()[0]])),
                     dtype=float)
        for i in xrange(self.nbVox):
            for j in xrange(self.nbConditions):
                tnrl[i, j, parmask[j]] = self.timeNrls[j][i, :]

        # TODO: dirac outputs to use less space
        axes_domains = {'condition': self.dataInput.cNames,
                        'time': arange(0, self.dataInput.paradigm.get_t_max(), dt)}
        outputs['pm_tNrls'] = xndarray(tnrl,
                                       axes_names=[
                                           'voxel', 'condition', 'time'],
                                       axes_domains=axes_domains,
                                       value_label="nrl")
        if self.keepSamples:
            axes_names = ['iteration', 'condition', 'voxel']
            axes_domains = {'classe': ['inactif', 'actif'],
                            'condition': self.dataInput.cNames,
                            'iteration': self.sampleHistoryIterations}
            outputs['pm_Habits_hist'] = xndarray(self.habitsHistory,
                                                 axes_names=axes_names,
                                                 axes_domains=axes_domains,
                                                 value_label="Habituation")

            if self.outputRatio:
                axes_names = ['iteration', 'condition', 'voxel', 'ratio']
                axes_domains = {'condition': self.dataInput.cNames,
                                'iteration': self.sampleHistoryIterations,
                                'ratio': [1, 2]}
                outputs['pm_ratio_hist'] = xndarray(self.ratioHistory,
                                                    axes_names=axes_names,
                                                    axes_domains=axes_domains,
                                                    value_label="Ratio")
                axes_names = [
                    'iteration', 'condition', 'voxel', 'courbe', 'ratio']
                axes_domains = {'condition': self.dataInput.cNames,
                                'iteration': self.sampleHistoryIterations,
                                'ratio': [1, 2, 3, 4, 5],
                                'courbe': arange(0., 1., 0.01)
                                }
                outputs['pm_ratiocourbe_hist'] = xndarray(self.ratiocourbeHistory,
                                                          axes_names=axes_names,
                                                          axes_domains=axes_domains,
                                                          value_label="Courbe Ratio")

                axes_names = ['condition', 'voxel']
                axes_domains = {'condition': self.dataInput.cNames}
                outputs['pm_compteur'] = xndarray(self.compteur,
                                                  axes_names=axes_names,
                                                  axes_domains=axes_domains,
                                                  value_label="compteur")

        return outputs

    def cleanMemory(self):

        # NRLSampler.cleanMemory(self)

        del self.varClassApost
        del self.sigClassApost
        del self.sigApost
        del self.meanApost
        del self.aa
        del self.aXh
        del self.varYtilde
        del self.varXhtQ
        del self.sumaXh
        if hasattr(self, 'labelsSamples'):
            del self.labelsSamples
        del self.corrEnergies
        del self.labels
        del self.voxIdx

        del self.sumaX
        #del self.sumaXQ
        del self.Xmask
        del self.nnulls
        del self.varSingleCondXtrials
        del self.voxelActivity

        del self.sumaXhtQaXh

        # clean memory of temporary variables :
        #del self.timeNrls
        del self.Gamma
        del self.habits

        del self.onsets
        del self.deltaOns
        del self.nbTrials

        #del self.outputRatio
        #del self.trueHabits
        del self.nh
        #del self.sumaXtQ
        #del self.sumaXtQaX

        del self.varXh
        #del self.varXhtQXh
        #del self.varsumXh

    def spExtract(self, spInd, mtrx, cond):
        X = mtrx + 0.
        xr = X.ravel()
        xr[argsort(xr)] = concatenate(
            (self.nnulls[cond], spInd.repeat(self.nbGammas[cond])), axis=0)
        return X

        # def bilinearsparsedot(self, gamma, Q, taille, cond):
        #Y = zeros((taille, taille), dtype= float)

        # for t1 in xrange(self.nbTrials[cond]):
        # for (k,i) in self.Xmask[cond][t1]:
        # for t2 in xrange(self.nbTrials[cond]):
        # for (l,j) in  self.Xmask[cond][t2]:
        #Y[i,j] += gamma[t1] * gamma[t2] * Q[k,l]
        # return Y

# il y a un probleme sur cette fonction


def sparsedot(X, A, mask, taille):
    Y = zeros(taille, dtype=float)
    # print shape(Y)
    # print shape(A)
    # raw_input()
    for (i, k) in mask:
        # print i,k
        Y[:, k] += X * A[:, i]
    return Y

# il y a un probleme aussi sur cette fonction


def sparsedotdimun(X, A, mask, lenght):
    Y = zeros(lenght, dtype=float)
    for (i, k) in mask:
        Y[i] += X * A[k]
    return Y


##----- functions for habitSamper() ------##

def LaplacianPdf(beta, r0Hab, a, b, N=1):  # instrumental law for MH algo
    if size(r0Hab) > 1:
        N = size(r0Hab)
        u = numpy.random.rand(1, N)
    elif size(r0Hab) == 1:
        u = numpy.random.rand()
    la = beta * a
    lb = beta * b
    lr0Hab = beta * r0Hab

    if all(r0Hab < a):
        #        print "r0Hab<a"
        abConst = math.exp(-la) - math.exp(-lb)
        x = -log(math.exp(-la) - u * abConst) / beta
        r0HabConst = 1.
    elif all(r0Hab > b):
        #        print "r0Hab>b"
        abConst = math.exp(lb) - math.exp(la)
        x = log(math.exp(la) + u * abConst) / beta
        r0HabConst = 1.
    elif size(r0Hab) == 1:  # r0Hab value in [a,b] interval
        #        print "size(r0Hab)==1"
        e_lamr0Hab = math.exp(beta * (a - r0Hab))
        e_lr0Habmb = math.exp(beta * (r0Hab - b))
        r0HabConst = (2. - e_lamr0Hab - e_lr0Habmb) / beta
        F_r0Hab = (1. - e_lamr0Hab) / (2. - e_lamr0Hab - e_lr0Habmb)
        if (u < F_r0Hab):
            abConst = r0HabConst * beta * math.exp(lr0Hab)
            x = log(math.exp(la) + u * abConst) / beta
        elif (u >= F_r0Hab):
            abConst = r0HabConst * beta
            x = r0Hab - log(2. - e_lamr0Hab - u * abConst) / beta
        else:
            abConst0 = r0HabConst * beta * math.exp(lr0Hab)
            abConst1 = r0HabConst * beta
#            for k in xrange(0,N):
            if (u < F_r0Hab):
                x = log(math.exp(la) + u * abConst0) / beta
            else:
                x = r0Hab - log(2. - e_lamr0Hab - u * abConst1) / beta
    else:
        x = zeros((N), dtype=float)
        for i in xrange(N):
            x[i] = LaplacianPdf(beta, r0Hab[i], a, b, 1.0)
            r0HabConst.append(r0HabConst)
    return x, r0HabConst

# def spExtract(spInd, mtrx):
    #X = mtrx + 0.
    # for j in xrange(spInd.size):
    #X[mtrx == (j+1)] = spInd[j]
    # return X

# def spExtract(spInd, mtrx, cond):
    #X = mtrx + 0.
    #xr = X.ravel()
    #xr[argsort(xr)] = concatenate(self.nnulls,spInd.repeat(self.nbGammas[cond]))
    # return X

# spExtract a bouge dans la class Habit


def subcptGamma(nrl, habit, nbTrials, deltaOns):
    # print 'function subcptGamma: start...',
    gamma = zeros(nbTrials, dtype=float)
    timeNrls = zeros(nbTrials, dtype=float)
    gamma[0] = 1.
    timeNrls[0] = nrl
    skk = 0.
    for k in xrange(1, nbTrials):
        skk = (habit ** deltaOns[k - 1]) * (skk + timeNrls[k - 1])
        gamma[k] = 1. / (1. + skk)
        timeNrls[k] = gamma[k] * nrl
    # print 'OK'
    # print 'skk',skk, 'gamma',gamma, 'timeNrls',timeNrls
#    return gamma
    return gamma, timeNrls

# def spExtract(spInd, mtrx):

    #X = array(mtrx, dtype=float) +0.

    # for j in xrange(spInd.size):
    #X[mtrx == (j+1)] = spInd[j]
    # return X


# def spExtract(vctr, mtrxX):
    ##x = mtrxX
    # v=(numpy.where(mtrxX!=0))
    # a=x[v]
    # kk=zeros(len(a))
    # for i in xrange(len(a)):
    # kk[i]=vctr[(a[i]-1)]
    # x[v]=kk
    # return x



# -*- coding: utf-8 -*-

import unittest
import shutil
from copy import deepcopy
import logging

import numpy as np

import pyhrf
import pyhrf.jde.asl as jasl

from pyhrf.core import FmriData
from pyhrf.ui.jde import JDEMCMCAnalyser
from pyhrf.ui.treatment import FMRITreatment


logger = logging.getLogger(__name__)


class ASLTest(unittest.TestCase):

    def setUp(self):
        np.random.seed(8652761)

        self.tmp_dir = pyhrf.get_tmp_path()
        self.clean_tmp = True

        self.sampler_params_for_single_test = {
            'nb_iterations': 40,
            'smpl_hist_pace': 1,
            'obs_hist_pace': 1,
            'brf': jasl.BOLDResponseSampler(do_sampling=False,
                                            normalise=1.,
                                            use_true_value=True,
                                            zero_constraint=False),
            'brf_var':
            jasl.BOLDResponseVarianceSampler(do_sampling=False,
                                             val_ini=np.array([.001]),
                                             use_true_value=False),
            'prf': jasl.PerfResponseSampler(do_sampling=False,
                                            normalise=1.,
                                            use_true_value=True,
                                            zero_constraint=False),
            'prf_var':
            jasl.PerfResponseVarianceSampler(do_sampling=False,
                                             val_ini=np.array([.001]),
                                             use_true_value=False),
            'noise_var': jasl.NoiseVarianceSampler(do_sampling=False,
                                                   use_true_value=True),
            'drift_var': jasl.DriftVarianceSampler(do_sampling=False,
                                                   use_true_value=True),
            'drift': jasl.DriftCoeffSampler(do_sampling=False,
                                            use_true_value=True),
            'bold_response_levels':
            jasl.BOLDResponseLevelSampler(do_sampling=False,
                                          use_true_value=True),
            'perf_response_levels':
            jasl.PerfResponseLevelSampler(do_sampling=False,
                                          use_true_value=True),
            'labels': jasl.LabelSampler(do_sampling=False,
                                        use_true_value=True),
            'bold_mixt_params': jasl.BOLDMixtureSampler(do_sampling=False,
                                                        use_true_value=True),
            'perf_mixt_params': jasl.PerfMixtureSampler(do_sampling=False,
                                                        use_true_value=True),
            'perf_baseline': jasl.PerfBaselineSampler(do_sampling=False,
                                                      use_true_value=True),
            'perf_baseline_var':
            jasl.PerfBaselineVarianceSampler(do_sampling=False,
                                             use_true_value=True),
            'check_final_value': 'raise',  # print or raise
        }

    def tearDown(self):
        if self.clean_tmp:
            logger.info('Remove tmp dir %s', self.tmp_dir)
            shutil.rmtree(self.tmp_dir)
        else:
            logger.info('Keep tmp dir %s', self.tmp_dir)

    def test_prf(self):
        """ Validate estimation of PRF """
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir)
        fdata = FmriData.from_simulation_dict(simu)
        self._test_specific_samplers(['prf'], fdata, nb_its=100,
                                     check_fv='raise')

    def test_brf(self):
        """ Validate estimation of BRF at high SNR"""
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir)
        fdata = FmriData.from_simulation_dict(simu)
        self._test_specific_samplers(['brf'], fdata, nb_its=100,
                                     check_fv='raise')

    def test_brls(self):
        """ Validate estimation of BRLs at high SNR"""
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir)
        fdata = FmriData.from_simulation_dict(simu)
        self._test_specific_samplers(['bold_response_levels'], fdata, nb_its=100,
                                     check_fv='raise')

    def test_prls(self):
        """ Validate estimation of PRLs at high SNR"""
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir)
        fdata = FmriData.from_simulation_dict(simu)
        self._test_specific_samplers(['perf_response_levels'], fdata, nb_its=100,
                                     check_fv='raise')

    def test_labels(self):
        """ Validate estimation of labels at high SNR"""
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir)
        fdata = FmriData.from_simulation_dict(simu)
        self._test_specific_samplers(['labels'], fdata, nb_its=100,
                                     check_fv='raise')

    def test_noise_var(self):
        """ Validate estimation of noise variances at high SNR"""
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir)
        fdata = FmriData.from_simulation_dict(simu)
        self._test_specific_samplers(['noise_var'], fdata, nb_its=100,
                                     check_fv='raise')

    def test_drift(self):
        """ Validate estimation of drift at high SNR"""
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir, spatial_size='normal')
        fdata = FmriData.from_simulation_dict(simu)
        self._test_specific_samplers(['drift'], fdata, nb_its=100,
                                     check_fv='print')

    def test_all(self):
        """ Validate estimation of full ASL model at high SNR"""
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        from pyhrf.jde.asl import simulate_asl
        simu = simulate_asl(self.tmp_dir, spatial_size='normal')
        fdata = FmriData.from_simulation_dict(simu)
        np.random.seed(25430)
        v = ['bold_response_levels', 'perf_response_levels', 'drift', 'drift_var',
             'brf', 'prf', 'labels', 'bold_mixt_params', 'perf_mixt_params',
             'perf_baseline', 'perf_baseline_var']

        self._test_specific_samplers(v, fdata, nb_its=100, check_fv='print')

    def _test_specific_samplers(self, sampler_names, fdata,
                                nb_its=None, use_true_val=None,
                                save_history=False, check_fv=None,
                                normalize_brf=1., normalize_prf=1.):
        """
        Test specific samplers.
        """
        if use_true_val is None:
            use_true_val = dict((n, False) for n in sampler_names)

        logger.info('_test_specific_samplers %s ...', str(sampler_names))

        params = deepcopy(self.sampler_params_for_single_test)

        # Loop over given samplers to enable them
        for var_name in sampler_names:
            var_class = params[var_name].__class__
            use_tval = use_true_val[var_name]

            # special case for HRF -> normalization and prior type
            if var_class == jasl.BOLDResponseSampler:
                params[var_name] = \
                    jasl.BOLDResponseSampler(do_sampling=True,
                                             use_true_value=use_tval,
                                             normalise=normalize_brf,
                                             zero_constraint=False)
            elif var_class == jasl.PerfResponseSampler:
                params[var_name] = \
                    jasl.PerfResponseSampler(do_sampling=True,
                                             use_true_value=use_tval,
                                             normalise=normalize_brf,
                                             zero_constraint=False)
            else:
                params[var_name] = var_class(do_sampling=True,
                                             use_true_value=use_tval)

        if nb_its is not None:
            params['nb_iterations'] = nb_its

        if save_history:
            params['smpl_hist_pace'] = 1
            params['obs_hist_pace'] = 1

        if check_fv is not None:
            params['check_final_value'] = check_fv

        sampler = jasl.ASLSampler(**params)

        output_dir = self.tmp_dir

        analyser = JDEMCMCAnalyser(sampler=sampler, osfMax=4, dtMin=.4,
                                   dt=fdata.simulation[0]['dt'], driftParam=4,
                                   driftType='polynomial',
                                   outputPrefix='jde_mcmc_',
                                   pass_error=False)

        treatment = FMRITreatment(fmri_data=fdata, analyser=analyser,
                                  output_dir=output_dir)

        outputs = treatment.run()
        print 'out_dir:', output_dir
        return outputs



# -*- coding: utf-8 -*-

import logging

import numpy.matlib

from numpy import *

from pyhrf.jde.samplerbase import *
from pyhrf.jde.beta import *
from pyhrf.jde.nrl.bigaussian import NRLSampler


logger = logging.getLogger(__name__)


class NRLARSampler(NRLSampler):
    """Class handling the Gibbs sampling of Neural Response Levels according to:

        Makni, S., Ciuciu, P., Idier, J., & Poline, J. (2006). Joint Detection-Estimation of Brain Activity in fMRI
        using an Autoregressive Noise Model. In 3rd IEEE International Symposium on Biomedical Imaging: Macro to Nano,
        2006. (pp. 1048–1051). IEEE. https://doi.org/10.1109/ISBI.2006.1625101

    Inherits the abstract class C{ GibbsSamplerVariable}.
    """

    def linkToData(self, dataInput):
        self.dataInput = dataInput
        self.nbConditions = self.dataInput.nbConditions
        self.nbVox = self.dataInput.nbVoxels
        self.ny = self.dataInput.ny
        self.cardClass = zeros((self.nbClasses, self.nbConditions), dtype=int)
        self.voxIdx = [range(self.nbConditions)
                       for _ in xrange(self.nbClasses)]

        self.cumulLabels = zeros((self.nbConditions, self.nbVox), dtype=int)
        self.meanClassAApost = empty((self.nbConditions, self.nbVox),
                                     dtype=float)
        self.meanClassIApost = empty((self.nbConditions, self.nbVox),
                                     dtype=float)
        self.varXjhtLambdaXjh = empty((self.nbVox), dtype=float)
        self.varXjhtLambdaeji = empty((self.nbVox), dtype=float)

    def samplingWarmUp(self, variables):
        """
        #TODO : comment
        """

        # Compute precalculations :
        smplHRF = self.get_variable('hrf')
        smplHRF.checkAndSetInitValue(variables)
        smpldrift = self.get_variable('drift')
        smpldrift.checkAndSetInitValue(variables)
        self.varYtilde = zeros((self.ny, self.nbVox), dtype=float)
        self.computeVarYTilde(smplHRF.varXh, smpldrift.varMBYPl)

        self.normHRF = smplHRF.norm
        shape = (self.nbClasses, self.nbConditions, self.nbVox)
        self.varClassApost = empty(shape, dtype=float)
        self.sigClassApost = empty(shape, dtype=float)
        self.meanClassApost = empty(shape, dtype=float)
        self.meanApost = empty((self.nbConditions, self.nbVox), dtype=float)
        self.sigApost = zeros((self.nbConditions, self.nbVox), dtype=float)

        self.corrEnergies = zeros((self.nbConditions, self.nbVox), dtype=float)

    def computeVarYTilde(self, varXh, varMBYPl):
        for i in xrange(self.nbVox):
            repNRLi = numpy.matlib.repmat(self.currentValue[:, i], self.ny, 1)
            aijXjh = repNRLi * varXh
            self.varYtilde[:, i] = varMBYPl[:, i] - aijXjh.sum(axis=1)

    def sampleNextAlt(self, variables):
        varXh = self.get_variable('hrf').varXh
        varMBYPl = self.get_variable('drift').varMBYPl
        self.computeVarYTilde(varXh, varMBYPl)

    def computeMeanVarClassApost(self, j, variables):
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        var = sIMixtP.getCurrentVars()
        mean = sIMixtP.getCurrentMeans()

        reps = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        varXh = self.get_variable('hrf').varXh
        varMBYPl = self.get_variable('drift').varMBYPl
        varInvAutoCorrNoise = (variables[self.samplerEngine.I_NOISE_ARP].
                               InvAutoCorrNoise)

        nrls = self.currentValue
        varXhj = varXh[:, j]

        logger.debug('varInvAutoCorrNoise[:,:,0] :')
        logger.debug(varInvAutoCorrNoise[:, :, 0])
        for i in xrange(self.nbVox):
            eji = self.varYtilde[:, i] + nrls[j, i] * varXhj
            varXjhtLambda = dot(varXhj.transpose(),
                                varInvAutoCorrNoise[:, :, i]) / reps[i]
            logger.debug('varXjhtLambda :')
            logger.debug(varXjhtLambda)

            self.varXjhtLambdaXjh = dot(varXjhtLambda, varXhj)
            self.varXjhtLambdaeji = dot(varXjhtLambda, eji)
            for c in xrange(self.nbClasses):
                self.varClassApost[c, j, i] = 1. / (1. / var[c, j] +
                                                    self.varXjhtLambdaXjh)
                self.varClassApost[
                    c, j, i] = self.sigClassApost[c, j, i] ** 0.5

                if c > 0:  # assume 0 stands for inactivating class
                    self.meanClassApost[c, j, i] = ((mean[c, j] / var[c, j] +
                                                     self.varXjhtLambdaeji) *
                                                    self.varClassApost[c, j, i])
                else:
                    self.meanClassApost[c, j, i] = (self.varClassApost[c, j, i] *
                                                    self.varXjhtLambdaeji)
            logger.debug('meanClassApost %d cond %d :', j, i)
            logger.debug(self.meanClassApost[:, j, i])

    def sampleNextInternal(self, variables):
        # TODO : comment

        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        varXh = self.get_variable('hrf').varXh
        varMBYPl = self.get_variable('drift').varMBYPl
        varInvAutoCorrNoise = (variables[self.samplerEngine.I_NOISE_ARP].
                               InvAutoCorrNoise)

        self.computeVarYTilde(varXh, varMBYPl)

        self.labelsSamples = random.rand(self.nbConditions, self.nbVox)
        nrlsSamples = random.randn(self.nbConditions, self.nbVox)

        for j in xrange(self.nbConditions):
            self.computeMeanVarClassApost(j, variables)
            logger.info('Sampling labels - cond %d ...', j)
            if self.sampleLabelsFlag:
                self.sampleLabels(j, variables)

            for c in xrange(self.nbClasses):
                putmask(self.sigApost[j, :], self.labels[j, :] == c,
                        self.sigClassApost[c, j, :])
                putmask(self.meanApost[j, :], self.labels[j, :] == c,
                        self.meanClassApost[c, j, :])

            add(multiply(nrlsSamples[j, :], self.sigApost[j, :]),
                self.meanApost[j, :],
                self.currentValue[j, :])

            logger.debug('All nrl cond %d:', j)
            logger.debug(self.currentValue[j, :])
            logger.info('nrl cond %d = %1.3f(%1.3f)',
                        j, self.currentValue[j, :].mean(),
                        self.currentValue[j, :].std())
            for c in xrange(self.nbClasses):
                logger.debug('All nrl %s cond %d:', self.CLASS_NAMES[c], j)
                ivc = self.voxIdx[c][j]
                logger.debug(self.currentValue[j, ivc])

                logger.info('nrl %s cond %d = %1.3f(%1.3f)',
                            self.CLASS_NAMES[c], j,
                            self.currentValue[j, ivc].mean(),
                            self.currentValue[j, ivc].std())

            self.computeVarYTilde(varXh, varMBYPl)

        self.countLabels(self.labels, self.voxIdx, self.cardClass)

    def cleanMemory(self):
        # clean memory of temporary variables :
        del self.varClassApost
        del self.sigClassApost
        del self.varXjhtLambdaeji
        del self.varXjhtLambdaXjh
        del self.sigApost
        del self.meanApost
        del self.varYtilde
        if hasattr(self, 'labelsSamples'):
            del self.labelsSamples
        del self.corrEnergies
        del self.labels
        del self.voxIdx



# -*- coding: utf-8 -*-

import unittest
import numpy as np

from pyhrf.boldsynth.spatialconfig import *
from pyhrf.boldsynth.field import *
from pyhrf.boldsynth.pottsfield.pottsfield_c import genPottsField
from pyhrf.graph import *
from pyhrf.stats.random import *
from pyhrf.boldsynth.scenarios import *


class Mapper1DTest(unittest.TestCase):

    def test3D(self):
        a = np.arange(2 * 2 * 2).reshape(2, 2, 2)
        mapping = np.array([[0, 0], [0, 1], [1, 0], [1, 1]])
        mapper = Mapper1D(mapping, a.shape[1:])
        fa = mapper.flattenArray(a, 1)
        efa = mapper.expandArray(fa, 1)
        assert (efa == a).all()

    def testIrregularMapping(self):
        a = np.arange(2 * 2 * 2).reshape(2, 2, 2)
        mapping = np.array([[1, 1], [0, 1], [1, 0], [0, 0]])
        mapper = Mapper1D(mapping, a.shape[1:])
        fa = mapper.flattenArray(a, 1)
        efa = mapper.expandArray(fa, 1)
        assert (efa == a).all()

    def testIncompleteMapping(self):
        a = np.arange(2 * 2 * 2).reshape(2, 2, 2)
        mapping = np.array([[1, 1], [0, 1], [0, 0]])
        mapper = Mapper1D(mapping, a.shape[1:])
        fa = mapper.flattenArray(a, 1)
        efa = mapper.expandArray(fa, 1, fillValue=-1)
        assert (efa[:, [1, 0, 0], [1, 1, 0]] ==
                a[:, [1, 0, 0], [1, 1, 0]]).all()


class FieldFuncsTest(unittest.TestCase):

    def test_count_homo_cliques(self):
        labels = np.array([[1, 1, 1],
                           [1, 1, 1],
                           [1, 1, 1]], dtype=int)
        mask = np.ones_like(labels)
        g = graph_from_lattice(mask, kerMask=kerMask2D_4n, toroidal=True)
        hc = count_homo_cliques(g, labels[np.where(mask)])
        assert hc == 2 * labels.size

    def test_count_homo_cliques1(self):
        labels = np.array([[0, 0, 1],
                           [0, 1, 1]], dtype=int)
        mask = np.ones_like(labels)
        g = graph_from_lattice(mask, kerMask=kerMask2D_4n)
        hc = count_homo_cliques(g, labels[np.where(mask)])
        assert hc == 4

    def test_count_homo_cliques2(self):
        labels = np.array([[0, 0, 1],
                           [1, 1, 1],
                           [1, 1, 1]], dtype=int)
        mask = np.ones_like(labels)
        g = graph_from_lattice(mask, kerMask=kerMask2D_4n)
        hc = count_homo_cliques(g, labels[np.where(mask)])
        assert hc == 9

    def test_swendsenwang(self):

        nbLabels = 2
        shape = (5, 5)
        mask = np.ones(shape, dtype=int)
        g = graph_from_lattice(mask, kerMask=kerMask2D_4n)
        labels = genPepperSaltField(mask.size, nbLabels)
        SwendsenWangSampler_graph(g, labels, 50.9, 2)
        mapPotts = np.ones_like(mask)
        mapPotts[np.where(mask)] = labels

    def test_potts_gibbs(self):
        nbLabels = 2
        shape = (15, 15)
        mask = np.ones(shape, dtype=int)
        g = graph_from_lattice(mask, kerMask=kerMask2D_4n)
        labels = genPepperSaltField(mask.size, nbLabels).astype(np.int32)
        toroidal = 1
        beta = .1
        genPottsField(shape[0], shape[1], toroidal, beta, 100000, labels,
                      nbLabels)
        mapPotts = np.ones_like(mask)
        mapPotts[np.where(mask)] = labels



# -*- coding: utf-8 -*-

import os
import os.path as op
import tempfile
import logging

import numpy as np

import pyhrf

from pyhrf.graph import graph_from_mesh, bfs_set_label, bfs_sub_graph
from pyhrf.tools import add_suffix, non_existent_file
from pyhrf.tools._io import (read_texture, write_texture, read_spatial_resolution,
                             read_volume, write_volume)


logger = logging.getLogger(__name__)


def mesh_contour(coords, triangles, labels):
    bridge_triangles = []
    selected_nodes = set()
    for tri in triangles:
        l = labels[tri[0]]
        if (labels[tri[1:]] != l).any():
            bridge_triangles.append(tri)
            selected_nodes.update(tri)

    selected_nodes = sorted(list(selected_nodes))
    reorder = np.zeros(len(coords))
    reorder[selected_nodes] = range(len(selected_nodes))
    bridge_triangles = [reorder[t] for t in bridge_triangles]

    return coords[selected_nodes], np.array(bridge_triangles, dtype=int)


def mesh_contour_with_files(input_mesh, input_labels, output_mesh=None,
                            output_labels=None):
    """ TODO: use nibabel here """
    #from gifti import loadImage, saveImage, GiftiDataArray, GiftiImage
    #from gifti import GiftiImage_fromarray, GiftiImage_fromTriangles
    #from gifti import GiftiIntentCode, GiftiEncoding

    labels = loadImage(input_labels).arrays[0].data.astype(int)
    cor, triangles = loadImage(input_mesh).arrays

    contour_cor, contour_tri = mesh_contour(cor.data, triangles.data.astype(int),
                                            labels)
    k = GiftiImage_fromarray(contour_cor)
    k.arrays[0].intentString = "NIFTI_INTENT_POINTSET"
    k.addDataArray_fromarray(
        contour_tri, GiftiIntentCode.NIFTI_INTENT_TRIANGLE)
    for a in k.arrays:
        a.encoding = GiftiEncoding.GIFTI_ENCODING_ASCII

    if output_mesh is None:
        output_mesh = non_existent_file(add_suffix(input_mesh, '_contour'))
    logger.info('saving to %s', output_mesh)
    k.save(output_mesh)


def extract_sub_mesh(cor, tri, center_node, radius):
    g = graph_from_mesh(tri)
    subg, kept_nodes = bfs_sub_graph(g, center_node, radius)
    subcor = cor[kept_nodes]
    n_idx = np.zeros(len(g), dtype=int)
    keep = np.zeros(len(g), dtype=bool)
    keep[kept_nodes] = True
    n_idx[kept_nodes] = range(len(kept_nodes))

    new_tri = np.vstack([n_idx[t] for t in tri if keep[t].all()])
    # new_tri = []
    # for t in tri.data:
    #     if keep[t].all():
    #         new_tri.append(n_idx[t])
    #new_tri = np.vstack(new_tri)

    return subcor, new_tri


def extract_sub_mesh_with_files(input_mesh, center_node, radius,
                                output_mesh=None):
    from nibabel import gifti
    from nibabel.gifti import GiftiImage, GiftiDataArray
    from pyhrf.tools._io import read_mesh
    cor, tri, coord_sys = read_mesh(input_mesh)
    sub_cor, sub_tri = extract_sub_mesh(cor, tri, center_node, radius)

    #nimg = GiftiImage_fromTriangles(sub_cor, sub_tri)
    nimg = GiftiImage()
    intent = 'NIFTI_INTENT_POINTSET'
    nimg.add_gifti_data_array(GiftiDataArray.from_array(sub_cor, intent))
    intent = 'NIFTI_INTENT_TRIANGLE'
    nimg.add_gifti_data_array(GiftiDataArray.from_array(sub_tri, intent))

    if output_mesh is None:
        output_mesh = non_existent_file(add_suffix(input_mesh, '_sub'))
    logger.info('Saving extracted mesh to %s', output_mesh)
    gifti.write(nimg, output_mesh)
    return sub_cor, sub_tri, coord_sys


# Projection onto surface

def create_projection_kernels(input_mesh, output_kernel, resolution,
                              geod_decay=5., norm_decay=2., size=7):
    logger.info('Create projection kernels (%s) ...', output_kernel)
    logger.info('Call AimsFunctionProjection -op 0 ...')

    projection = ['AimsFunctionProjection',
                  '-m', input_mesh,
                  '-o', output_kernel,
                  '-i', size,
                  #'-I', 801,
                  #'--debugLevel', 10,
                  #'--verbose', 10,
                  '-vx', resolution[0],
                  '-vy', resolution[1],
                  '-vz', resolution[2],
                  '-g', geod_decay,
                  '-n', norm_decay,
                  '-op', '0',
                  '-t', 0
                  ]
    os.system(' '.join(map(str, projection)))


def project_fmri(input_mesh, data_file, output_tex_file,
                 output_kernels_file=None, data_resolution=None,
                 geod_decay=5., norm_decay=2., kernel_size=7,
                 tex_bin_threshold=None):

    if output_kernels_file is None:
        tmp_dir = tempfile.mkdtemp(prefix='pyhrf_surf_proj',
                                   dir=pyhrf.cfg['global']['tmp_path'])

        kernels_file = op.join(tmp_dir, add_suffix(op.basename(data_file),
                                                   '_kernels'))
        tmp_kernels_file = True
    else:
        kernels_file = output_kernels_file
        tmp_kernels_file = False

    if data_resolution is not None:
        resolution = data_resolution
    else:
        resolution = read_spatial_resolution(data_file)

    logger.info('Data resolution: %s', resolution)
    logger.info('Projection parameters:')
    logger.info('   - geodesic decay: %f mm', geod_decay)
    logger.info('   - normal decay: %f mm', norm_decay)
    logger.info('   - kernel size: %f voxels', kernel_size)

    create_projection_kernels(input_mesh, kernels_file, resolution,
                              geod_decay, norm_decay, kernel_size)

    project_fmri_from_kernels(input_mesh, kernels_file, data_file,
                              output_tex_file, tex_bin_threshold)

    if tmp_kernels_file:
        os.remove(kernels_file)


def project_fmri_from_kernels(input_mesh, kernels_file, fmri_data_file,
                              output_tex, bin_threshold=None, ):

    logger.info('Project data onto mesh using kernels ...')

    if 0:
        print 'Projecting ...'
        print 'func data:', fmri_data_file
        print 'Mesh file:', input_mesh
        print 'Save as:', output_tex

    logger.info('Call AimsFunctionProjection -op 1 ...')

    data_files = []
    output_texs = []
    p_ids = None
    if bin_threshold is not None:
        d, h = read_volume(fmri_data_file)
        if np.allclose(d.astype(int), d):
            tmp_dir = pyhrf.get_tmp_path()
            p_ids = np.unique(d)
            logger.info('bin threshold: %f', bin_threshold)
            logger.info(
                'pids(n=%d): %d...%d', len(p_ids), min(p_ids), max(p_ids))
            for i, p_id in enumerate(p_ids):
                if p_id != 0:
                    new_p = np.zeros_like(d)
                    new_p[np.where(d == p_id)] = i + 1  # 0 is background
                    ifn = op.join(tmp_dir, 'pmask_%d.nii' % p_id)
                    write_volume(new_p, ifn, h)
                    data_files.append(ifn)
                    ofn = op.join(tmp_dir, 'ptex_%d.gii' % p_id)
                    output_texs.append(ofn)
        else:
            data_files.append(fmri_data_file)
            output_texs.append(output_tex)
    else:
        data_files.append(fmri_data_file)
        output_texs.append(output_tex)

    logger.info('input data files: %s', str(data_files))
    logger.info('output data files: %s', str(output_texs))

    for data_file, o_tex in zip(data_files, output_texs):
        projection = [
            'AimsFunctionProjection',
            '-op', '1',
            '-d', kernels_file,
            '-d1', data_file,
            '-m', input_mesh,
            '-o', o_tex
        ]

        cmd = ' '.join(map(str, projection))
        logger.info('cmd: %s', cmd)
        os.system(cmd)

    if bin_threshold is not None:
        logger.info('Binary threshold of texture at %f', bin_threshold)
        o_tex = output_texs[0]
        data, data_gii = read_texture(o_tex)
        data = (data > bin_threshold).astype(np.int32)
        print 'data:', data.dtype
        if p_ids is not None:
            for pid, o_tex in zip(p_ids[1:], output_texs[1:]):
                pdata, pdata_gii = read_texture(o_tex)
                data += (pdata > bin_threshold).astype(np.int32) * pid

        #assert (np.unique(data) == p_ids).all()
        write_texture(data, output_tex, intent='NIFTI_INTENT_LABEL')



# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil
import logging

from copy import deepcopy

import numpy as np

import pyhrf

from pyhrf import FmriData
from pyhrf.jde.models import BOLDGibbsSampler as BG
from pyhrf.jde.beta import BetaSampler as BS
from pyhrf.jde.nrl.bigaussian import NRLSampler as NS
from pyhrf.jde.nrl.bigaussian import BiGaussMixtureParamsSampler as BGMS
from pyhrf.jde.hrf import RHSampler as HVS
from pyhrf.jde.hrf import HRFSampler as HS
from pyhrf.jde.models import simulate_bold
from pyhrf.jde.noise import NoiseVarianceSampler
from pyhrf.jde.samplerbase import GibbsSamplerVariable


logger = logging.getLogger(__name__)


class JDETest(unittest.TestCase):

    def setUp(self):
        np.random.seed(8652761)

        tmpDir = tempfile.mkdtemp(prefix='pyhrf_tests',
                                  dir=pyhrf.cfg['global']['tmp_path'])
        self.tmp_dir = tmpDir
        self.clean_tmp = True

        bf = 'subj0_bold_session0.nii.gz'
        self.boldFiles = [pyhrf.get_data_file_name(bf)]
        pf = 'subj0_parcellation.nii.gz'
        self.parcelFile = pyhrf.get_data_file_name(pf)
        self.tr = 2.4
        self.dt = .6
        self.onsets = pyhrf.onsets_loc_av
        #self.durations = pyhrf.durations_loc_av
        self.durations = None
        self.nbIt = 3
        self.pfMethod = 'es'

        simu = simulate_bold(self.tmp_dir, spatial_size='normal')
        self.data_simu = FmriData.from_simulation_dict(simu)

        self.tmp_dir_small = tempfile.mkdtemp(prefix='small_simu',
                                              dir=self.tmp_dir)
        simu = simulate_bold(self.tmp_dir_small, spatial_size='small')
        self.data_small_simu = FmriData.from_simulation_dict(simu)

        print 'Create sampler_params_for_single_test ...'
        self.sampler_params_for_single_test = {
            BG.P_NB_ITERATIONS: 100,
            BG.P_SMPL_HIST_PACE: 1,
            BG.P_OBS_HIST_PACE: 1,
            # level of spatial correlation = beta
            BG.P_BETA: BS({
                BS.P_SAMPLE_FLAG: False,
                BS.P_USE_TRUE_VALUE: False,
                BS.P_VAL_INI: np.array([0.6]),
            }),
            # HRF
            BG.P_HRF: HS({
                HS.P_SAMPLE_FLAG: False,
                HS.P_USE_TRUE_VALUE: True,
                HS.P_PRIOR_TYPE: 'singleHRF',
            }),
            # HRF variance
            BG.P_RH: HVS({
                HVS.P_USE_TRUE_VALUE: True,
                HVS.P_SAMPLE_FLAG: False,
            }),
            # neural response levels (stimulus-induced effects)
            BG.P_NRLS: NS({
                NS.P_USE_TRUE_NRLS: True,
                NS.P_USE_TRUE_LABELS: True,
                NS.P_SAMPLE_FLAG: False,
                NS.P_SAMPLE_LABELS: False,
            }),
            BG.P_MIXT_PARAM: BGMS({
                BGMS.P_SAMPLE_FLAG: False,
                BGMS.P_USE_TRUE_VALUE: True,
            }),
            BG.P_NOISE_VAR: NoiseVarianceSampler({
                NoiseVarianceSampler.P_SAMPLE_FLAG: False,
                NoiseVarianceSampler.P_USE_TRUE_VALUE: True,
            }),
            BG.P_CHECK_FINAL_VALUE: 'raise',  # print or raise
        }

        print 'Create sampler_params_for_full_test ...'
        self.sampler_params_for_full_test = {
            BG.P_NB_ITERATIONS: 500,
            BG.P_SMPL_HIST_PACE: 1,
            BG.P_OBS_HIST_PACE: 1,
            # level of spatial correlation = beta
            BG.P_BETA: BS({
                BS.P_SAMPLE_FLAG: True,
                BS.P_USE_TRUE_VALUE: False,
                BS.P_VAL_INI: np.array([0.6]),
            }),
            # HRF
            BG.P_HRF: HS({
                HS.P_SAMPLE_FLAG: True,
                HS.P_USE_TRUE_VALUE: False,
                HS.P_NORMALISE: 1.,
            }),
            # HRF variance
            BG.P_RH: HVS({
                HVS.P_USE_TRUE_VALUE: False,
                HVS.P_SAMPLE_FLAG: True,
            }),
            # neural response levels (stimulus-induced effects)
            BG.P_NRLS: NS({
                NS.P_USE_TRUE_NRLS: False,
                NS.P_USE_TRUE_LABELS: False,
                NS.P_SAMPLE_FLAG: True,
                NS.P_SAMPLE_LABELS: True,
            }),
            BG.P_MIXT_PARAM: BGMS({
                BGMS.P_SAMPLE_FLAG: True,
                BGMS.P_USE_TRUE_VALUE: False,
            }),
            BG.P_NOISE_VAR: NoiseVarianceSampler({
                NoiseVarianceSampler.P_SAMPLE_FLAG: True,
                NoiseVarianceSampler.P_USE_TRUE_VALUE: False,
            }),
            BG.P_CHECK_FINAL_VALUE: 'print',  # print or raise
        }

    def tearDown(self):
        if self.clean_tmp:
            logger.info('Remove tmp dir %s', self.tmp_dir)
            shutil.rmtree(self.tmp_dir)
        else:
            logger.info('Keep tmp dir %s', self.tmp_dir)

    def _test_specific_samplers(self, sampler_names, simu_scenario='small',
                                nb_its=None, use_true_val=None,
                                use_true_nrlsbar=False, use_true_labels=False,
                                sample_labels=True, sample_nrlsbar=True,
                                save_history=False, check_fv=None,
                                normalize_hrf=1., hrf_prior_type='singleHRF'):

        if use_true_val is None:
            use_true_val = dict((n, False) for n in sampler_names)

        logger.info('_test_specific_samplers %s ...', str(sampler_names))

        params = deepcopy(self.sampler_params_for_single_test)

        for var_name in sampler_names:
            var_class = params[var_name].__class__

            if var_class == HS:
                dict_hrf = {
                    HS.P_SAMPLE_FLAG: True,
                    HS.P_USE_TRUE_VALUE:
                    use_true_val[var_name],
                    HS.P_NORMALISE: normalize_hrf,
                    HS.P_PRIOR_TYPE: hrf_prior_type,
                }
                params[var_name] = var_class(dict_hrf)
            elif var_class == NS:
                dict_nrl = {
                    var_class.P_SAMPLE_FLAG: sample_nrlsbar,
                    var_class.P_USE_TRUE_NRLS: use_true_nrlsbar,
                    var_class.P_SAMPLE_LABELS: sample_labels,
                    var_class.P_USE_TRUE_LABELS: use_true_labels,
                }
                params[var_name] = var_class(dict_nrl)
            else:
                dict_var = {var_class.P_SAMPLE_FLAG: True,
                            var_class.P_USE_TRUE_VALUE: use_true_val[var_name],
                            }
                params[var_name] = var_class(dict_var)

        for var_name, param in params.iteritems():
            if isinstance(param, GibbsSamplerVariable):
                print 'param[%s] -> use_true_val: %s' \
                    % (var_name, param.useTrueValue)

        if nb_its is not None:
            params[BG.P_NB_ITERATIONS] = nb_its

        if save_history:
            params[BG.P_SMPL_HIST_PACE] = 1
            params[BG.P_OBS_HIST_PACE] = 1

        if check_fv is not None:
            params[BG.P_CHECK_FINAL_VALUE] = check_fv

        if BG.P_HRF not in sampler_names:
            dict_hrf = {
                HS.P_SAMPLE_FLAG: False,
                HS.P_USE_TRUE_VALUE: True,
                HS.P_NORMALISE: normalize_hrf,
                HS.P_PRIOR_TYPE: hrf_prior_type,
            }
            params[BG.P_HRF] = HS(dict_hrf)

        sampler = BG(params)

        if simu_scenario == 'small':
            simu = self.data_small_simu
            output_dir = self.tmp_dir_small
        elif simu_scenario == 'normal':
            simu = self.data_simu
            output_dir = self.tmp_dir
        else:
            simu = simu_scenario
            output_dir = self.tmp_dir

        analyser = JDEMCMCAnalyser(sampler=sampler, osfMax=4, dtMin=.4,
                                   dt=.5, driftParam=4, driftType='polynomial',
                                   outputPrefix='jde_mcmc_',
                                   randomSeed=5421087, pass_error=False)

        treatment = FMRITreatment(fmri_data=simu, analyser=analyser,
                                  output_dir=output_dir)

        treatment.run()
        print 'out_dir:', output_dir

    def test_noise_var_sampler(self):
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)
        self._test_specific_samplers(['noiseVariance'],
                                     check_fv='raise')

    def test_hrf_var_sampler(self):

        # estimation of HRF variance tested in the following situation:
        #    -  simulated gaussian smooth HRF is not normalized
        #       -> else the simulated HRF variance is not consistent

        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)

        simu = simulate_bold(self.tmp_dir, spatial_size='small',
                             normalize_hrf=False)
        simu = FmriData.from_simulation_dict(simu)

        self._test_specific_samplers(['HRFVariance'], simu_scenario=simu,
                                     check_fv='raise', nb_its=100,
                                     hrf_prior_type='singleHRF')

    def test_hrf_var_sampler_2(self):

        # estimation of HRF variance tested in the following situation:
        #    -  simulated gaussian smooth HRF is not normalized
        #       -> else the simulated HRF variance is not consistent

        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)

        simu = simulate_bold(self.tmp_dir, spatial_size='small',
                             normalize_hrf=False)
        simu = FmriData.from_simulation_dict(simu)

        self._test_specific_samplers(['HRFVariance'], simu_scenario=simu,
                                     check_fv='raise', nb_its=100,
                                     hrf_prior_type='voxelwiseIID')

    def test_hrf_with_var_sampler(self):
        # estimation of HRF and its variance tested in the following situation:
        #    -  simulated gaussian smooth HRF is not normalized

        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)

        simu = simulate_bold(self.tmp_dir, spatial_size='small',
                             normalize_hrf=False)
        simu = FmriData.from_simulation_dict(simu)

        self._test_specific_samplers(['HRFVariance', 'HRF'], simu_scenario=simu,
                                     check_fv='print', nb_its=100,
                                     hrf_prior_type='singleHRF')

    def test_hrf_with_var_sampler_2(self):
        # estimation of HRF and its variance tested in the following situation:
        #    -  simulated gaussian smooth HRF is not normalized

        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)

        simu = simulate_bold(self.tmp_dir, spatial_size='small',
                             normalize_hrf=False)
        simu = FmriData.from_simulation_dict(simu)

        self._test_specific_samplers(['HRFVariance', 'HRF'], simu_scenario=simu,
                                     check_fv='print', nb_its=100,
                                     hrf_prior_type='voxelwiseIID')

    def test_full_sampler(self):
        """ Test JDE on simulation with normal size.
        Estimation accuracy is tested.
        """
        # pyhrf.verbose.set_verbosity(2)
        pyhrf.logger.setLevel(logging.INFO)

        simu = simulate_bold(self.tmp_dir, spatial_size='normal',
                             normalize_hrf=False)
        simu = FmriData.from_simulation_dict(simu)

        sampler = BG(self.sampler_params_for_full_test)

        analyser = JDEMCMCAnalyser(sampler=sampler, osfMax=4, dtMin=.4,
                                   dt=.5, driftParam=4, driftType='polynomial',
                                   outputPrefix='jde_mcmc_',
                                   randomSeed=None)

        treatment = FMRITreatment(fmri_data=simu,
                                  analyser=analyser, output_dir=self.tmp_dir)

        treatment.run()
        print 'output_dir:', self.tmp_dir



# -*- coding: utf-8 -*-


from math import *
#import collections
from pyhrf.tools import apply_to_leaves
import re
from copy import deepcopy
"""
Mathematical Expression Evaluator class for Python.
You can set the expression member, set the functions, variables and then call
evaluate() function that will return you the result of the mathematical expression
given as a string. 
 
The user is granted rights to user, distribute and/or modify the source code 
as long as this notice is shipped with it.
 
The Author of this software cannot and do not warrant that any 
functions contained in the Software will meet your requirements, 
or that its operations will be error free. 
 
The entire risk as to the Software performance or quality, 
or both, is solely with the user and not the Author.
 
You assume responsibility for the selection of the software to 
achieve your intended results, and for the installation, use, and 
results obtained from the Software. 
 
The Author makes no warranty, either implied or expressed, including 
with-out limitation any warranty with respect to this Software 
documented here, its quality, performance, or fitness for a particular 
purpose. In no event shall the Author be liable to you for damages, 
whether direct or indirect, incidental, special, or consequential 
arising out the use of or any defect in the Software, even if the 
Author has been advised of the possibility of such damages, 
or for any claim by any other party. 
 
All other warranties of any kind, either express or implied, 
including but not limited to the implied warranties of 
merchantability and fitness for a particular purpose, are expressly 
excluded.
 
Copyright and all rights of this software, irrespective of what
has been deposited with the U.S. Copyright Office, belongs
to Bestcode.com.
"""

class ArithmeticExpressionSyntaxError(Exception):
    pass

class ArithmeticExpressionNameError(Exception):
    pass


class ArithmeticExpression(object):
    '''
    Mathematical Expression Evaluator class.
    You can set the expression member, set the functions, variables and then call
    evaluate() function that will return you the result of the mathematical 
    expression given as a string.
    '''
   
    '''
    Dictionary of functions that can be used in the expression.
    '''
    functions = None #{'__builtins__':None};
   
    '''
    Dictionary of variables that can be used in the expression.
    '''
    variables = None #{'__builtins__':None};

    def __init__(self, expression, **variables):
        ''' Constructor '''
        self.expression = expression

        if self.variables is None:
            self.variables = {'__builtins__':None}

        if self.functions is None:
            functions = {'__builtins__':None};

        for vname, val in variables.iteritems():
            self.variables[vname] = val
   
    def call_if_func(self, x):
        if hasattr(x, '__call__'): # or isinstance(x, collections.Callable):
            return x()
        else: return x

    def evaluate(self):
        '''
        Evaluate the mathematical expression given as a string in the 
        expression member variable.
       
        '''
        variables = apply_to_leaves(self.variables, self.call_if_func)
        try:
            result = eval(self.expression, variables, self.functions)
        except NameError, err:
            undef_var = re.findall("name '(.*)' is not defined", err.args[0])[0]
            raise NameError("Variable '%s' is not defined in expression '%s'" \
                                %(undef_var, self.expression), undef_var)
        
        return result


    def check(self):
        
        #print 'self.variables:', self.variables
        try:
            self.evaluate()
        except Exception ,err :
            m = 'Error in expression "%s": ' %self.expression
            if isinstance(err, NameError):
                exception_occurs = True
                undefs = []
                old_vars = self.variables.copy()
                while exception_occurs:
                    exception_occurs = False
                    try:
                        self.evaluate()
                    except NameError, err :
                        exception_occurs = True
                        v = self.variables
                        undefs.append(err.args[1])
                        if len(v) == 1: # only default __builtins__
                            dummy_val = 1
                        elif v.keys()[0] == '__builtins__':
                            dummy_val = v[v.keys()[1]]
                        else:
                            dummy_val = v[v.keys()[0]]
                        v[err.args[1]] = dummy_val
                m += ' *undefined variable(s): %s' %','.join(undefs)
                self.variables = old_vars
                raise ArithmeticExpressionNameError(m, self.expression,
                                                    undefs)
            elif isinstance(err, SyntaxError):
                m += ' *syntax error at offset %d' %err.offset
                raise ArithmeticExpressionSyntaxError(m, self.expression,
                                                      err.offset)
            else:
                raise err

    def addDefaultFunctions(self):
        '''
        Add the following Python functions to be used in a mathemtical expression:
        acos
        asin
        atan
        atan2
        ceil
        cos
        cosh
        degrees
        exp
        fabs
        floor
        fmod
        frexp
        hypot
        ldexp
        log
        log10
        modf
        pow
        radians
        sin
        sinh
        sqrt
        tan
        tanh
        '''
        self.functions['acos']=acos
        self.functions['asin']=asin
        self.functions['atan']=atan
        self.functions['atan2']=atan2
        self.functions['ceil']=ceil
        self.functions['cos']=cos
        self.functions['cosh']=cosh
        self.functions['degrees']=degrees
        self.functions['exp']=exp
        self.functions['fabs']=fabs
        self.functions['floor']=floor
        self.functions['fmod']=fmod
        self.functions['frexp']=frexp
        self.functions['hypot']=hypot
        self.functions['ldexp']=ldexp
        self.functions['log']=log
        self.functions['log10']=log10
        self.functions['modf']=modf
        self.functions['pow']=pow
        self.functions['radians']=radians
        self.functions['sin']=sin
        self.functions['sinh']=sinh
        self.functions['sqrt']=sqrt
        self.functions['tan']=tan
        self.functions['tanh']=tanh
       
    def addDefaultVariables(self):
        '''
        Add e and pi to the list of defined variables.
        '''
        self.variables['e']=e
        self.variables['pi']=pi

    def setVariable(self, name, value):
        '''
        Define the value of a variable defined by name
        '''
        self.variables[name] = value

    def getVariableNames(self):
        '''
        Return a List of defined variables names in sorted order.
        '''
        mylist = list(self.variables.keys())
        try:
            mylist.remove('__builtins__')
        except ValueError:
            pass
        mylist.sort()
        return mylist


    def getFunctionNames(self):
        '''
        Return a List of defined function names in sorted order.
        '''
        mylist = list(self.functions.keys())
        try:
            mylist.remove('__builtins__')
        except ValueError:
            pass
        mylist.sort()
        return mylist



# -*- coding: utf-8 -*-

import logging

from numpy import *
from scipy.integrate import trapz

import pyhrf
from pyhrf.ndarray import xndarray
from pyhrf.jde.intensivecalc import calcCorrEnergies
from pyhrf.jde.samplerbase import *
from pyhrf.jde.beta import *
from pyhrf.jde.nrl.bigaussian import NRLSampler, BiGaussMixtureParamsSampler


logger = logging.getLogger(__name__)


class GGGNRLSampler(NRLSampler):

    L_CI = 0
    L_CA = 1
    L_CD = 2
    CLASSES = array([L_CI, L_CA, L_CD], dtype=int)
    CLASS_NAMES = NRLSampler.CLASS_NAMES + ['deactiv']
    FALSE_POS = 3
    FALSE_NEG = 4

    def sampleLabels(self, cond, variables):  # varCI, varCA, meanCA):
        print 'GGGNRLSampler ... sampleLabels'
        sMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        varCI = sMixtP.currentValue[sMixtP.I_VAR_CI]
        varCA = sMixtP.currentValue[sMixtP.I_VAR_CA]
        meanCA = sMixtP.currentValue[sMixtP.I_MEAN_CA]
        meanCD = sMixtP.currentValue[sMixtP.I_MEAN_CD]
        varCD = sMixtP.currentValue[sMixtP.I_VAR_CD]

        fracLambdaTildeI_A = self.calcFracLambdaTilde(cond, self.L_CI, self.L_CA,
                                                      variables)
        fracLambdaTildeD_A = self.calcFracLambdaTilde(cond, self.L_CD, self.L_CA,
                                                      variables)
        fracLambdaTildeA_D = 1 / fracLambdaTildeD_A
        fracLambdaTildeI_D = self.calcFracLambdaTilde(cond, self.L_CI, self.L_CD,
                                                      variables)

        if self.beta[cond] > 0:
            if 1:
                dColI_A = 0
                dColD_A = 0
                dColA_D = 0  # -dColD_A
                dColI_D = 0
                beta = self.beta[cond]
                # TODO : optimize : do it in one C function call !!
                calcCorrEnergies(cond, self.labels, self.corrEnergies,
                                 self.dataInput.neighboursIndexes,
                                 dColI_A, self.nbClasses, self.L_CI, self.L_CA)
                fracLambdaTildeI_A *= exp(beta * self.corrEnergies[cond, :])
                calcCorrEnergies(cond, self.labels, self.corrEnergies,
                                 self.dataInput.neighboursIndexes,
                                 dColD_A, self.nbClasses, self.L_CD, self.L_CA)
                fracLambdaTildeD_A *= exp(beta * self.corrEnergies[cond, :])

                calcCorrEnergies(cond, self.labels, self.corrEnergies,
                                 self.dataInput.neighboursIndexes,
                                 dColA_D, self.nbClasses, self.L_CA, self.L_CD)
                fracLambdaTildeA_D *= exp(beta * self.corrEnergies[cond, :])

                calcCorrEnergies(cond, self.labels, self.corrEnergies,
                                 self.dataInput.neighboursIndexes,
                                 dColI_D, self.nbClasses, self.L_CI, self.L_CD)
                fracLambdaTildeI_D *= exp(beta * self.corrEnergies[cond, :])

            else:
                for i in xrange(self.nbVox):
                    deltaE = self.calcDeltaEnergy(i, cond)
                    fracLambdaTilde[i] *= exp(-self.beta[cond] * deltaE)
        varLambdaApost_A = 1 / (1 + fracLambdaTildeI_A + fracLambdaTildeD_A)
        varLambdaApost_D = 1 / (1 + fracLambdaTildeI_D + fracLambdaTildeA_D)

        smpl = self.labelsSamples[cond, :]
        ivD = (smpl <= varLambdaApost_D)
        self.labels[cond, ivD] = self.L_CD
        ivA = bitwise_and(smpl > varLambdaApost_D,
                          smpl <= varLambdaApost_A + varLambdaApost_D)
        self.labels[cond, ivA] = self.L_CA
        self.labels[
            cond, smpl > varLambdaApost_A + varLambdaApost_D] = self.L_CI


class TriGaussMixtureParamsSampler(BiGaussMixtureParamsSampler):

    I_MEAN_CD = 3
    I_VAR_CD = 4
    NB_PARAMS = 5
    PARAMS_NAMES = ['Mean_Activ', 'Var_Activ', 'Var_Inactiv',
                    'Mean_Deactiv', 'Var_Deactiv']

    P_MEAN_CD_PR_MEAN = 'meanCDPrMean'
    P_MEAN_CD_PR_VAR = 'meanCDPrVar'

    P_VAR_CD_PR_ALPHA = 'varCDPrAlpha'
    P_VAR_CD_PR_BETA = 'varCDPrBeta'

    L_CD = GGGNRLSampler.L_CD

    def __init__(self, do_sampling=True, use_true_value=False,
                 val_ini=None, hyper_prior_type='Jeffreys', activ_thresh=4.,
                 var_ci_pr_alpha=2.04, var_ci_pr_beta=.5,
                 var_ca_pr_alpha=2.01, var_ca_pr_beta=.5,
                 var_cd_pr_alpha=2.01, var_cd_pr_beta=.5,
                 mean_ca_pr_mean=5., mean_ca_pr_var=20.,
                 mean_cd_pr_mean=-20., mean_cd_pr_var=20.):

        BiGaussMixtureParamsSampler.__init__(self, do_sampling, use_true_value,
                                             val_ini, hyper_prior_type,
                                             activ_thresh, var_ci_pr_alpha,
                                             var_ci_pr_beta, var_ca_pr_alpha,
                                             var_ca_pr_beta, mean_ca_pr_mean,
                                             mean_ca_pr_var)
        self.varCDPrAlpha = var_cd_pr_alpha
        self.varCDPrBeta = var_cd_pr_beta
        self.meanCDPrMean = mean_cd_pr_mean
        self.meanCDPrVar = mean_cd_pr_var

    def linkToData(self, dataInput):
        BiGaussMixtureParamsSampler.linkToData(self, dataInput)
        self.nrlCD = range(self.nbConditions)
        if self.dataInput.simulData is not None:
            mixtures = self.dataInput.simulData.nrls.getMixture()
            itemsCond = mixtures.items()
            meanCD = zeros(self.nbConditions, dtype=float)
            varCD = zeros(self.nbConditions, dtype=float)
            for cn, mixt in mixtures.iteritems():
                genDeactiv = mixt.generators['deactiv']
                indCond = self.dataInput.simulData.nrls.condIds[cn]
                meanCD[indCond] = genDeactiv.mean
                varCD[indCond] = genDeactiv.std ** 2
            self.trueValue[self.I_MEAN_CD] = meanCD
            self.trueValue[self.I_VAR_CD] = varCD

    # TODO: generalize inside parent class
    def checkAndSetInitValue(self, variables):

        if self.currentValue is None:
            curValWasNone = True
        else:
            curValWasNone = False
        BiGaussMixtureParamsSampler.checkAndSetInitValue(self, variables)
        # TODO : retrieve simulated components ...

        if curValWasNone:
            if not self.useTrueValue:
                nc = self.nbConditions
                self.currentValue[self.I_MEAN_CD] = zeros(nc) - 10.
                self.currentValue[self.I_VAR_CD] = zeros(nc) + 1.

    def getCurrentVars(self):
        return array([self.currentValue[self.I_VAR_CI],
                      self.currentValue[self.I_VAR_CA],
                      self.currentValue[self.I_VAR_CD]])

    def getCurrentMeans(self):
        return array([zeros(self.nbConditions), self.currentValue[self.I_MEAN_CA],
                      self.currentValue[self.I_MEAN_CD]])

    def computeWithJeffreyPriors(self, j, cardCDj):
        logger.info('cond %d - card CD = %d', j, cardCDj)
        logger.info('cond %d - cur mean CD = %f', j,
                    self.currentValue[self.I_MEAN_CD, j])
        if cardCDj > 0:
            logger.info('cond %d - nrl CD: %f(v%f)[%f,%f]', j,
                        self.nrlCD[j].mean(), self.nrlCD[j].var(),
                        self.nrlCD[j].min(), self.nrlCD[j].max())

        if cardCDj > 1:
            nrlC2Centered = self.nrlCD[j] - \
                self.currentValue[self.I_MEAN_CD, j]
            nu2j = dot(nrlC2Centered, nrlC2Centered)
            varCDj = 1.0 / random.gamma(0.5 * (cardCDj + 1) - 1, 2. / nu2j)

            eta2j = mean(self.nrlCD[j])
            meanCDj = random.normal(eta2j, (varCDj / cardCDj) ** 0.5)

            logger.info('varCD ~ InvGamma(%f, nu2j/2=%f)', 0.5 * (cardCDj + 1) - 1,
                        nu2j / 2.)
            logger.info(
                ' -> mean = %f', (nu2j / 2.) / (0.5 * (cardCDj + 1) - 1))
        else:
            logger.info('Warning : cardCD <= 1!')
            varCDj = 1.0 / random.gamma(.5, 2.)
            if cardCDj == 0:
                meanCDj = random.normal(5.0, varCDj ** 0.5)
            else:
                meanCDj = random.normal(self.nrlCD[j], varCDj ** 0.5)

        logger.info('Sampled components - cond: %d', j)
        logger.info('mean CD = %f, var CD = %f', meanCDj, varCDj)

        return meanCDj, varCDj

    def sampleNextInternal(self, variables):
        nrlsSmpl = variables[self.samplerEngine.I_NRLS]
        cardCD = nrlsSmpl.cardClass[self.L_CD, :]

        if self.hyperPriorFlag:
            for j in xrange(self.nbConditions):
                vICD = nrlsSmpl.voxIdx[nrlsSmpl.L_CD][j]
                self.nrlCD[j] = nrlsSmpl.currentValue[j, vICD]

                if cardCD[j] > 0:
                    etaj = mean(self.nrlCD[j])
                    nrlCDCentered = self.nrlCD[j] - etaj
                    nuj = .5 * dot(nrlCDCentered, nrlCDCentered)
                    varCDj = 1.0 / random.gamma(0.5 * cardCD[j] + self.varCDPrAlpha,
                                                1 / (nuj + self.varCDPrBeta))
                else:
                    etaj = 0.0
                    varCDj = 1.0 / \
                        random.gamma(self.varCDPrAlpha, 1 / self.varCDPrBeta)

                invVarLikelihood = cardCD[j] / varCDj
                meanCDVarAPost = 1 / (invVarLikelihood + 1 / self.meanCDPrVar)
                rPrMV = self.meanCDPrMean / self.meanCDPrVar
                meanCDMeanAPost = meanCDVarAPost * \
                    (etaj * invVarLikelihood + rPrMV)
                meanCDj = random.normal(meanCDMeanAPost, meanCDVarAPost ** 0.5)

                self.currentValue[self.I_MEAN_CD, j] = meanCDj
                self.currentValue[self.I_VAR_CD, j] = varCDj

        else:
            nrlsSmpl = variables[self.samplerEngine.I_NRLS]

            for j in random.permutation(self.nbConditions):
                ca = nrlsSmpl.cardClass[self.L_CA, j]
                ci = nrlsSmpl.cardClass[self.L_CI, j]
                cd = nrlsSmpl.cardClass[self.L_CD, j]
                vICI = nrlsSmpl.voxIdx[nrlsSmpl.L_CI][j]
                vICA = nrlsSmpl.voxIdx[nrlsSmpl.L_CA][j]
                vICD = nrlsSmpl.voxIdx[nrlsSmpl.L_CD][j]
                self.nrlCI[j] = nrlsSmpl.currentValue[j, vICI]
                self.nrlCA[j] = nrlsSmpl.currentValue[j, vICA]
                self.nrlCD[j] = nrlsSmpl.currentValue[j, vICD]

                r = BiGaussMixtureParamsSampler.computeWithJeffreyPriors(self, j,
                                                                         ci, ca)
                varCIj, meanCAj, varCAj = r
                meanCD, varCD = self.computeWithJeffreyPriors(j, cd)

                self.currentValue[self.I_VAR_CI, j] = varCIj
                # absolute(meanCAj)
                self.currentValue[self.I_MEAN_CA, j] = meanCAj
                self.currentValue[self.I_VAR_CA, j] = varCAj
                self.currentValue[self.I_MEAN_CD, j] = meanCD
                self.currentValue[self.I_VAR_CD, j] = varCD

                logger.info('meanCD,%d = %f', j,
                            self.currentValue[self.I_MEAN_CD, j])

                logger.info(
                    'varCD,%d = %f', j, self.currentValue[self.I_VAR_CD, j])

    def finalizeSampling(self):
        BiGaussMixtureParamsSampler.finalizeSampling(self)
        del self.nrlCD

    def getOutputs(self):
        outputs = GibbsSamplerVariable.getOutputs(self)
        mixtp = zeros((3, self.nbConditions, 2))
        mixtp[self.L_CA, :, 0] = self.finalValue[self.I_MEAN_CA, :]
        mixtp[self.L_CA, :, 1] = self.finalValue[self.I_VAR_CA, :]
        mixtp[self.L_CI, :, 0] = 0.
        mixtp[self.L_CI, :, 1] = self.finalValue[self.I_VAR_CI, :]
        mixtp[self.L_CD, :, 0] = self.finalValue[self.I_MEAN_CD, :]
        mixtp[self.L_CD, :, 1] = self.finalValue[self.I_VAR_CD, :]
        an = ['class', 'condition', 'component']
        ad = {'class': ['inactiv', 'activ', 'deactiv'],
              'condition': self.dataInput.cNames,
              'component': ['mean', 'var']}
        outputs[
            'pm_' + self.name] = xndarray(mixtp, axes_names=an, axes_domains=ad)

        return outputs



# -*- coding: utf-8 -*-
# Copyright CEA and IFR 49 (2000-2005)
#
#  This software and supporting documentation were developed by
#      CEA/DSV/SHFJ and IFR 49
#      4 place du General Leclerc
#      91401 Orsay cedex
#      France
#
# This software is governed by the CeCILL license version 2 under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the
# terms of the CeCILL license version 2 as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license version 2 and that you accept its terms.

'''
This package provide a mean to print colored message to a standard terminal if
color is available else message are print in black and white mode. If stdout
is redirected in a file or piped to an other program, the output is made
black and white to avoid issus with strange caracters that defined colors
in terminals. Remember that all messages are print to stdout.

This package exists in 3 places, for some very good arguments :
     datamind.tools.message
     soma.wip.message
     pyhrf.tools.message

To use these functionalities, play with 'msg' instance. Here, some classical
uses :

    msg.info('something cool happend'):
    msg.error(self, 'too bad, an error'):
    msg.warning(self, 'something strange but not fatal'):
    msg.write_list(('no color', ('color in red', 'red'))):
    msg.write('simple colored write function')
    msg.string('string to colored string')

'''

import sys

# add your shell colors here :
colors = {
		'gray' : '\033[01;30m',
		'bold_red' : '\033[01;31m',
		'bold_green' : '\033[01;32m',
		'bold_yellow' : '\033[01;33m',
		'bold_blue' : '\033[01;34m',
		'bold_purple' : '\033[01;35m',
		'bold_cyan' : '\033[01;36m',
		'bold_white' : '\033[01;37m',
		'black' : '\033[02;30m',
		'dark_red' : '\033[02;31m',
		'dark_green' : '\033[02;32m',
		'dark_yellow' : '\033[02;33m',
		'dark_blue' : '\033[02;34m',
		'dark_purple' : '\033[02;35m',
		'dark_cyan' : '\033[02;36m',
		'dark_gray' : '\033[02;37m',
		'red' : '\033[03;31m',
		'green' : '\033[03;32m',
		'yellow' : '\033[03;33m',
		'blue' : '\033[03;34m',
		'purple' : '\033[03;35m',
		'cyan' : '\033[03;36m',
		'gray' : '\033[03;37m',
		'underline_red' : '\033[04;31m',
		'underline_green' : '\033[04;32m',
		'underline_yellow' : '\033[04;33m',
		'underline_blue' : '\033[04;34m',
		'underline_purple' : '\033[04;35m',
		'underline_cyan' : '\033[04;36m',
		'underline_gray' : '\033[04;37m',
		'underline_white' : '\033[04;38m',
		'thin_red' : '\033[05;31m',
		'thin_green' : '\033[05;32m',
		'thin_yellow' : '\033[05;33m',
		'thin_blue' : '\033[05;34m',
		'thin_purple' : '\033[05;35m',
		'thin_cyan' : '\033[05;36m',
		'thin_gray' : '\033[05;37m',
		'invert_red' : '\033[07;31m',
		'invert_green' : '\033[07;32m',
		'invert_yellow' : '\033[07;33m',
		'invert_blue' : '\033[07;34m',
		'invert_purple' : '\033[07;35m',
		'invert_cyan' : '\033[07;36m',
		'invert_gray' : '\033[07;37m',
		'invert_white' : '\033[07;38m',
		'cross_red' : '\033[09;31m',
		'cross_green' : '\033[09;32m',
		'cross_yellow' : '\033[09;33m',
		'cross_blue' : '\033[09;34m',
		'cross_purple' : '\033[09;35m',
		'cross_cyan' : '\033[09;36m',
		'cross_gray' : '\033[09;37m',
		'cross_white' : '\033[09;38m',
		'back' : '\x1b[00m'}


class MessageColor(object):
	def haveColor(self):
		return True

	def info(self, msg):
		print ' *', colors['bold_green'] + 'Info' + \
				colors['back'], ':', msg

	def error(self, msg):
		sys.stderr.write(' * ' + colors['bold_red'] + 'Error' + \
			colors['back'] + ' : ' + msg + '\n')

	def warning(self, msg):
		print ' *', colors['bold_yellow'] + 'Warning' + \
			colors['back'], ':', msg

	def write_list(self, msg_list):
		for l in msg_list :
			if type(l) in [list, tuple]:
				self.write(*l)
			else:	self.write(l)

	def write(self, msg, color = 'back'):
		sys.stdout.write(self.string(msg, color))

	def string(self, msg, color = 'back'):
		if color[0] == '\033':
			c = color
		else:	c = colors[color]
		return c + str(msg) + colors['back']

	string = classmethod(string)
	write = classmethod(write)
	write_list = classmethod(write_list)
	error = classmethod(error)
	warning = classmethod(warning)
	info = classmethod(info)

class MessageNoColor(MessageColor):
	def haveColor(self):
		return False

	def info(self, msg):
		print ' * Info :', msg

	def error(self, msg):
		sys.stderr.write(' * Error : ' + msg + '\n')

	def warning(self, msg):
		print ' * Warning :', msg

	def string(self, msg, color = 'back'):
		return str(msg)

	string = classmethod(string)
	error = classmethod(error)
	warning = classmethod(warning)
	info = classmethod(info)

class Message(object):
	import sys

	# FIXME : add settings check for colors here
	if sys.stdout.isatty():
		_msg = MessageColor
	else:	_msg = MessageNoColor

	def __new__(cls, *args, **kwargs):
		return Message._msg.__new__(Message._msg, *args, **kwargs)

class NoMessage(object):
	def haveColor(self):
		return False

	def info(self, msg): pass
	def error(self, msg): pass
	def warning(self, msg): pass
	def string(self, msg, color = 'back'): pass
	def write(self, msg, color = 'back'): pass
	def write_list(self, msg_list): pass



msg = Message()



# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil
import logging

import numpy as np

import pyhrf
import pyhrf.tools as tools
import pyhrf.vbjde.vem_tools as vt

from pyhrf import FmriData
from pyhrf.jde.models import simulate_bold
from pyhrf.boldsynth.hrf import getCanoHRF
try:
    from collections import OrderedDict
except ImportError:
    from pyhrf.tools.backports import OrderedDict


logger = logging.getLogger(__name__)


class VEMToolsTest(unittest.TestCase):

    def setUp(self):
        np.random.seed(8652761)

        tmpDir = tempfile.mkdtemp(prefix='pyhrf_tests',
                                  dir=pyhrf.cfg['global']['tmp_path'])
        self.tmp_dir = tmpDir
        simu = simulate_bold(self.tmp_dir, spatial_size='random_small')
        self.data_simu = FmriData.from_simulation_dict(simu)
        self.clean_tmp = True

    def tearDown(self):
        if self.clean_tmp:
            logger.info('Remove tmp dir %s', self.tmp_dir)
            shutil.rmtree(self.tmp_dir)
        else:
            logger.info('Keep tmp dir %s', self.tmp_dir)

    def test_free_energy(self):
        """ Test of vem tool to compute free energy """
        M = 51
        D = 3
        N = 325
        J = 25
        K = 2
        TR = 1.
        Thrf = 25.
        dt = .5
        gamma_h = 1000
        data = self.data_simu
        Y = data.bold
        graph = data.get_graph()
        onsets = data.paradigm.get_joined_onsets()
        durations = data.paradigm.stimDurations
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        y_tilde = Y - np.dot(P, L)
        TT, m_h = getCanoHRF(Thrf, dt)
        order = 2
        D2 = vt.buildFiniteDiffMatrix(order, D)
        R = np.dot(D2, D2) / pow(dt, 2*order)
        invR = np.linalg.inv(R)
        Det_invR = np.linalg.det(invR)
        q_Z = 0.5 * np.ones((M, K, J), dtype=np.float64)
        neighbours_indexes = vt.create_neighbours(graph)
        Beta = np.ones((M), dtype=np.float64)
        sigma_epsilone = np.ones(J)
        _, occurence_matrix, _ = vt.create_conditions(onsets, durations, M, N, D, TR, dt)
        Gamma = np.identity(N)
        Det_Gamma = np.linalg.det(Gamma)
        XGamma = np.zeros((M, D, N), dtype=np.float64)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        mu_M = np.zeros((M, K), dtype=np.float64)
        sigma_M = np.ones((M, K), dtype=np.float64)
        m_H = np.array(m_h[:D]).astype(np.float64)
        Sigma_H = np.ones((D, D), dtype=np.float64)
        free_energy = vt.free_energy_computation(m_A, Sigma_A, m_H, Sigma_H, D,
                                                 q_Z, y_tilde, occurence_matrix,
                                                 sigma_epsilone, Gamma, M, J, N,
                                                 K, mu_M, sigma_M, neighbours_indexes,
                                                 Beta, Sigma_H, np.linalg.inv(R),
                                                 R, Det_Gamma, gamma_h)

    def test_computeFit(self):
        X = OrderedDict([])
        M = 51
        N = 325
        J = 25
        Thrf = 25.
        dt = .5
        TT, m_h = getCanoHRF(Thrf, dt)
        m_A = np.zeros((J, M), dtype=np.float64)
        m_H = np.array(m_h).astype(np.float64)
        stimIndSignal = vt.computeFit(m_H, m_A, X, J, N)

    def test_entropyA(self):
        M = 51
        J = 25
        Sigma_A = np.zeros((M, M, J), np.float64)
        entropy = vt.nrls_entropy(Sigma_A, M)

    def test_entropyH(self):
        D = 3
        Sigma_H = np.ones((D, D), dtype=np.float64)
        entropy = vt.hrf_entropy(Sigma_H, D)

    def test_entropyZ(self):
        M = 51
        K = 2
        J = 25
        q_Z = np.zeros((M, K, J), dtype=np.float64)
        entropy = vt.labels_entropy(q_Z)

    def test_max_mu_sigma(self):
        M = 51
        K = 2
        J = 25
        q_Z = np.zeros((M, K, J), dtype=np.float64)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        sigma_M = np.ones((M, K), dtype=np.float64)
        sigma_M[:, 0] = 0.1
        sigma_M[:, 1] = 1.0
        mu_M = np.zeros((M, K), dtype=np.float64)
        Mu, Sigma = vt.maximization_class_proba(q_Z, m_A, Sigma_A)

    def test_max_L(self):
        M = 51
        N = 325
        J = 25
        TR = 1.
        Thrf = 25.
        dt = .5
        TT, m_h = getCanoHRF(Thrf, dt)
        m_A = np.zeros((J, M), dtype=np.float64)
        m_H = np.array(m_h).astype(np.float64)
        data = self.data_simu
        onsets = data.paradigm.get_joined_onsets()
        durations = data.paradigm.get_joined_durations()
        Y = data.bold
        X = OrderedDict([])
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        zerosP = np.zeros((P.shape[0]), dtype=np.float64)
        _, occurence_matrix, _ = vt.create_conditions(onsets, durations, M, N,
                                                      len(m_H), TR, dt)
        L = vt.maximization_drift_coeffs(Y, m_A, occurence_matrix, m_H, np.identity(N), P)

    def test_max_sigmaH(self):
        D = 3
        Thrf = 25.
        dt = .5
        TT, m_h = getCanoHRF(Thrf, dt)
        m_h = m_h[:D]
        m_H = np.array(m_h).astype(np.float64)
        Sigma_H = np.ones((D, D), dtype=np.float64)
        order = 2
        D2 = vt.buildFiniteDiffMatrix(order, D)
        R = np.dot(D2, D2) / pow(dt, 2*order)
        sigmaH = vt.maximization_sigmaH(D, Sigma_H, R, m_H)

    def test_max_sigmaH_prior(self):
        D = 3
        Thrf = 25.
        dt = .5
        TT, m_h = getCanoHRF(Thrf, dt)
        m_h = m_h[:D]
        m_H = np.array(m_h).astype(np.float64)
        Sigma_H = np.ones((D, D), dtype=np.float64)
        order = 2
        D2 = vt.buildFiniteDiffMatrix(order, D)
        R = np.dot(D2, D2) / pow(dt, 2*order)
        gamma_h = 1000
        sigmaH = vt.maximization_sigmaH_prior(D, Sigma_H, R, m_H, gamma_h)

    def test_max_sigma_noise(self):
        M = 51
        N = 325
        J = 25
        TR = 1.
        Thrf = 25.
        dt = .5
        data = self.data_simu
        X = OrderedDict([])
        Y = data.bold
        onsets = data.get_joined_onsets()
        durations = data.paradigm.stimDurations
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        PL = np.dot(P, L)
        TT, m_h = getCanoHRF(Thrf, dt)
        sigma_epsilone = np.ones(J)
        Gamma = np.identity(N)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        m_H = np.array(m_h).astype(np.float64)
        D = len(m_H)
        Sigma_H = np.ones((D, D), dtype=np.float64)
        zerosMM = np.zeros((M, M), dtype=np.float64)
        _, occurence_matrix, _ = vt.create_conditions(onsets, durations, M, N,
                                                      D, TR, dt)
        sigma_eps = vt.maximization_noise_var(occurence_matrix, m_H, Sigma_H, m_A, Sigma_A,
                                              Gamma, Y, N)

    def test_expectZ(self):
        M = 51
        J = 25
        K = 2
        data = self.data_simu
        graph = data.get_graph()
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        mu_M = np.zeros((M, K), dtype=np.float64)
        sigma_M = np.ones((M, K), dtype=np.float64)
        beta = .8
        Beta = beta * np.ones((M), dtype=np.float64)
        q_Z = np.zeros((M, K, J), dtype=np.float64)
        Z_tilde = q_Z.copy()
        zerosK = np.zeros(K)
        neighbours_indexes = vt.create_neighbours(graph)
        q_Z = vt.labels_expectation(Sigma_A, m_A, sigma_M, mu_M, Beta, q_Z,
                                    neighbours_indexes, M, K)

    def test_expectH(self):
        M = 51
        K = 2
        J = 25
        N = 325
        D = 3
        TR = 1.
        Thrf = 25.
        dt = .5
        data = self.data_simu
        Gamma = np.identity(N)
        X = OrderedDict([])
        Y = data.bold
        onsets = data.get_joined_onsets()
        durations = data.paradigm.stimDurations
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        PL = np.dot(P, L)
        y_tilde = Y - np.dot(P, L)
        TT, m_h = getCanoHRF(Thrf, dt)
        m_h = m_h[:D]
        m_H = np.array(m_h)
        sigma_epsilone = np.ones(J)
        Sigma_H = np.ones((D, D), dtype=float)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        scale = 1
        zerosDD = np.zeros((D, D), dtype=np.float64)
        zerosD = np.zeros((D), dtype=np.float64)
        zerosND = np.zeros((N, D), dtype=np.float64)
        order = 2
        D2 = vt.buildFiniteDiffMatrix(order, D)
        R = np.dot(D2, D2) / pow(dt, 2*order)
        sigmaH = 0.1
        _, occurence_matrix, _ = vt.create_conditions(onsets, durations, M, N,
                                                      D, TR, dt)
        m_H, Sigma_H = vt.hrf_expectation(Sigma_A, m_A, occurence_matrix, Gamma, R,
                                          sigmaH, J, y_tilde, sigma_epsilone)

    def test_expectA(self):
        M = 51
        K = 2
        J = 25
        N = 325
        D = 3
        TR = 1.
        Thrf = 25.
        dt = .5
        data = self.data_simu
        Y = data.bold
        Onsets = data.get_joined_onsets()
        durations = data.paradigm.stimDurations
        Gamma = np.identity(N)
        X = OrderedDict([])
        for condition, Ons in Onsets.iteritems():
            X[condition] = vt.compute_mat_X_2(N, TR, D, dt, Ons)
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        PL = np.dot(P, L)
        y_tilde = Y - np.dot(P, L)
        TT, m_h = getCanoHRF(Thrf, dt)
        m_h = m_h[:D]
        sigma_epsilone = np.ones(J)
        m_H = np.array(m_h)
        Sigma_H = np.ones((D, D), dtype=float)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.ones((M, M, J), np.float64)
        for j in xrange(0, J):
            Sigma_A[:, :, j] = 0.01*np.identity(M)
        mu_M = np.zeros((M, K), dtype=np.float64)
        sigma_M = np.ones((M, K), dtype=np.float64)
        q_Z = 0.5 * np.ones((M, K, J), dtype=np.float64)
        zerosJMD = np.zeros((J, M, D), dtype=np.float64)
        _, occurence_matrix, _ = vt.create_conditions(Onsets, durations, M, N,
                                                      D, TR, dt)
        m_A, Sigma_A = vt.nrls_expectation(m_H, m_A, occurence_matrix, Gamma,
                                           q_Z, mu_M, sigma_M, M, y_tilde, Sigma_A,
                                           Sigma_H, sigma_epsilone)

    def test_matrix(self):
        Thrf = 25.
        dt = .5
        TT, m_h = getCanoHRF(Thrf, dt)
        m_H = np.array(m_h)
        m = vt.mult(m_H, m_H)

    def test_maximum(self):
        M = 51
        K = 2
        vector = np.ones((K, 1), dtype=np.float64)
        vector[0] = 2
        maxs, maxs_ind = vt.maximum(vector)

    def test_normpdf(self):
        y = vt.normpdf(1, 1, 1)

    def test_polyFit(self):
        N = 325
        TR = 1.
        data = self.data_simu
        Y = data.bold
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)

    def test_PolyMat(self):
        N = 325
        TR = 1.
        P = vt.PolyMat(N, 4, TR)

    def test_compute_mat_X2(self):
        N = 325
        TR = 1.
        D = 3
        dt = .5
        data = self.data_simu
        X = OrderedDict([])
        Onsets = data.get_joined_onsets()
        for condition, Ons in Onsets.iteritems():
            X[condition] = vt.compute_mat_X_2(N, TR, D, dt, Ons)

    def test_create_neighbours(self):
        graph = self.data_simu.get_graph()
        neighbours_indexes = vt.create_neighbours(graph)

    def test_create_conditions(self):
        nb_conditions = self.data_simu.nbConditions
        nb_scans = 325
        hrf_len = 3
        tr = 1.
        dt = .5
        onsets = self.data_simu.get_joined_onsets()
        durations = self.data_simu.paradigm.stimDurations
        X, occurence_matrix, condition_names = vt.create_conditions(
            onsets, durations, nb_conditions, nb_scans, hrf_len, tr, dt
        )

    def test_buildFiniteDiffMatrix(self):
        order = 2
        D = 3
        diffMat = vt.buildFiniteDiffMatrix(order, D)

    def test_gradient(self):
        beta = .8
        gamma = 7.5
        m = 0
        J = 25
        K = 2
        M = 51
        data = self.data_simu
        graph = data.get_graph()
        q_Z = np.zeros((M, K, J), dtype=np.float64)
        neighbours_indexes = vt.create_neighbours(graph)
        labels_neigh = vt.sum_over_neighbours(neighbours_indexes, q_Z)
        Gr = vt.beta_gradient(beta, q_Z, labels_neigh, neighbours_indexes, gamma)

    def test_max_beta(self):
        beta = .8
        gamma = 7.5
        m = 0
        J = 25
        K = 2
        M = 51
        data = self.data_simu
        graph = data.get_graph()
        q_Z = np.zeros((M, K, J), dtype=np.float64)
        neighbours_indexes = vt.create_neighbours(graph)
        beta = vt.beta_maximization(beta, q_Z, neighbours_indexes, gamma)



# -*- coding: utf-8 -*-

import unittest
import os
import cPickle
import os.path as op
import shutil

import numpy.testing as npt

import pyhrf
import pyhrf.ui.treatment as ptr
import pyhrf.paradigm as ppar

from pyhrf.configuration import cfg
from pyhrf import tools


class CmdInputTest(unittest.TestCase):
    """
    Test extraction of information from the command line to create an
    FmriTreatment
    """

    def setUp(self):
        self.tmp_dir = pyhrf.get_tmp_path()

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def _test_spm_option_parse(self, spm_ver):
        """
        Test parsing of option "-s SPM.mat" with given SPM version (int)
        """
        spm_file = op.join(self.tmp_dir, 'SPM_v{}.mat'.format(spm_ver))
        tools._io._zip.gunzip(pyhrf.get_data_file_name('SPM_v%d.mat.gz' % spm_ver),
                              outFileName=spm_file)

        options = ['-s', spm_file]
        from optparse import OptionParser
        parser = OptionParser()
        ptr.append_common_treatment_options(parser)

        fd = ptr.parse_data_options(parser.parse_args(options)[0])

        self.assertEqual(fd.tr, 2.4)  # nb sessions
        p = fd.paradigm
        # nb sessions
        self.assertEqual(len(p.stimOnsets[p.stimOnsets.keys()[0]]), 2)
        npt.assert_almost_equal(p.stimOnsets['audio'][0],
                                ppar.onsets_loc_av['audio'][0])
        npt.assert_almost_equal(p.stimOnsets['audio'][1],
                                ppar.onsets_loc_av['audio'][0])
        npt.assert_almost_equal(p.stimOnsets['video'][1],
                                ppar.onsets_loc_av['video'][0])

    def test_spm5_option_parse(self):
        """
        Test parsing of option "-s SPM.mat" (SPM5)
        """
        self._test_spm_option_parse(5)

    def test_spm8_option_parse(self):
        """
        Test parsing of option "-s SPM.mat" (SPM8)
        """
        self._test_spm_option_parse(8)

    def test_spm12_option_parse(self):
        """
        Test parsing of option "-s SPM.mat" (SPM12)
        """
        self._test_spm_option_parse(12)


class TreatmentTest(unittest.TestCase):

    def setUp(self):
        self.tmp_dir = pyhrf.get_tmp_path()

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_default_treatment(self):

        t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None)
        t.enable_draft_testing()
        t.run()

    @unittest.skipIf(not tools.is_importable('joblib'),
                     'joblib (optional dep) is N/A')
    def test_parallel_local(self):

        t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None)
        t.enable_draft_testing()
        t.run(parallel='local', n_jobs=2)

    def test_pickle_treatment(self):
        t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None)
        t.enable_draft_testing()
        cPickle.loads(cPickle.dumps(t))

    def test_sub_treatment(self):

        t = ptr.FMRITreatment(output_dir=self.tmp_dir)
        t.enable_draft_testing()
        sub_ts = t.split()
        for sub_t in sub_ts:
            sub_t.run()

    def test_jde_estim_from_treatment_pck(self):

        t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None)
        t.enable_draft_testing()
        sub_ts = t.split()
        sub_t_fn = op.join(self.tmp_dir, 'treatment.pck')
        fout = open(sub_t_fn, 'w')
        cPickle.dump(sub_ts[0], fout)
        fout.close()
        cmd = 'pyhrf_jde_estim  -t %s' % sub_t_fn
        if os.system(cmd) != 0:
            raise Exception('"' + cmd + '" did not execute correctly')

    @unittest.skipIf(not tools.is_importable('joblib'),
                     'joblib (optional dep) is N/A')
    def test_default_treatment_parallel_local(self):
        t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None)
        t.enable_draft_testing()
        t.run(parallel='local')

    @unittest.skipIf(not tools.is_importable('joblib'),
                     'joblib (optional dep) is N/A')
    def test_default_jde_cmd_parallel_local(self):
        t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None)
        t.enable_draft_testing()
        t_fn = op.join(self.tmp_dir, 'treatment.pck')
        fout = open(t_fn, 'w')
        cPickle.dump(t, fout)
        fout.close()
        cmd = 'pyhrf_jde_estim -t %s -x local' % t_fn
        if os.system(cmd) != 0:
            raise Exception('"' + cmd + '" did not execute correctly')

    def test_default_treatment_parallel_LAN(self):
        if cfg['parallel-LAN']['enable_unit_test'] == 1:
            t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None,
                                  output_dir=self.tmp_dir)
            t.enable_draft_testing()
            t.run(parallel='LAN')
        else:
            print 'LAN testing is off '\
                '([parallel-LAN][enable_unit_test] = 0 in ~/.pyhrf/config.cfg'

    def test_remote_dir_writable(self):
        if cfg['parallel-LAN']['enable_unit_test'] == 1:
            from pyhrf import grid
            lhosts = cfg['parallel-LAN']['hosts'].split(',')
            res = grid.remote_dir_is_writable(cfg['parallel-LAN']['user'],
                                              lhosts,
                                              cfg['parallel-LAN']['remote_path'])
            bad_hosts = [h for r, h in zip(res, cfg['parallel-LAN']['hosts'])
                         if r == 'no']

            if len(bad_hosts) > 0:
                raise Exception('Remote dir %s is not writable from the '
                                'following hosts:\n %s' % '\n'.join(bad_hosts))

        else:
            print 'LAN testing is off '\
                '([parallel-LAN][enable_unit_test] = 0 in config.cfg'

    def test_default_treatment_parallel_cluster(self):
        if cfg['parallel-cluster']['enable_unit_test'] == 1:
            t = ptr.FMRITreatment(make_outputs=False, result_dump_file=None,
                                  output_dir=self.tmp_dir)
            t.enable_draft_testing()
            t.run(parallel='cluster')
        else:
            print 'Cluster testing is off '\
                '([cluster-LAN][enable_unit_test] = 0 in config.cfg'



# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil
import logging

import numpy as np

import pyhrf
import pyhrf.tools as tools
import pyhrf.vbjde.vem_tools as vt

from pyhrf import FmriData
from pyhrf.vbjde import UtilsC
from pyhrf.ui.treatment import FMRITreatment
from pyhrf.jde.models import simulate_bold
from pyhrf.boldsynth.hrf import getCanoHRF
try:
    from collections import OrderedDict
except ImportError:
    from pyhrf.tools.backports import OrderedDict


logger = logging.getLogger(__name__)


class VEMToolsTest(unittest.TestCase):

    def setUp(self):
        np.random.seed(8652761)

        tmpDir = tempfile.mkdtemp(prefix='pyhrf_tests',
                                  dir=pyhrf.cfg['global']['tmp_path'])
        self.tmp_dir = tmpDir
        self.clean_tmp = True
        simu = simulate_bold(self.tmp_dir, spatial_size='random_small')
        self.data_simu = FmriData.from_simulation_dict(simu)

    def tearDown(self):
        if self.clean_tmp:
            logger.info('Remove tmp dir %s', self.tmp_dir)
            shutil.rmtree(self.tmp_dir)
        else:
            logger.info('Keep tmp dir %s', self.tmp_dir)

    def test_max_L(self):
        M = 51
        N = 325
        J = 25
        D = 3
        TR = 1.
        Thrf = 25.
        dt = .5
        TT, m_h = getCanoHRF(Thrf, dt)
        m_A = np.zeros((J, M), dtype=np.float64)
        m_H = np.array(m_h).astype(np.float64)
        data = self.data_simu
        Y = data.bold
        XX = np.zeros((M, N, D), dtype=np.int32)
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        Ndrift = L.shape[0]
        zerosP = np.zeros((P.shape[0]), dtype=np.float64)
        UtilsC.maximization_L(
            Y, m_A, m_H, L, P, XX.astype(np.int32), J, D, M, Ndrift, N)

    def test_max_sigma_noise(self):
        M = 51
        D = 3
        N = 325
        J = 25
        TR = 1.
        Thrf = 25.
        dt = .5
        Gamma = np.identity(N)
        data = self.data_simu
        XX = np.zeros((M, N, D), dtype=np.int32)
        Y = data.bold
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        PL = np.dot(P, L)
        TT, m_h = getCanoHRF(Thrf, dt)
        sigma_epsilone = np.ones(J)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        m_H = np.array(m_h).astype(np.float64)
        Sigma_H = np.ones((D, D), dtype=np.float64)
        UtilsC.maximization_sigma_noise(Gamma, PL, sigma_epsilone,
                                        Sigma_H, Y, m_A, m_H, Sigma_A,
                                        XX.astype(np.int32), J, D, M, N)

    def test_expectZ(self):
        M = 51
        J = 25
        K = 2
        data = self.data_simu
        graph = data.get_graph()
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        mu_M = np.zeros((M, K), dtype=np.float64)
        sigma_M = np.ones((M, K), dtype=np.float64)
        beta = .8
        Beta = beta * np.ones((M), dtype=np.float64)
        q_Z = np.zeros((M, K, J), dtype=np.float64)
        Z_tilde = q_Z.copy()
        maxNeighbours = max([len(nl) for nl in graph])
        neighboursIndexes = np.zeros((J, maxNeighbours), dtype=np.int32)
        neighboursIndexes -= 1
        for i in xrange(J):
            neighboursIndexes[i, :len(graph[i])] = graph[i]
        UtilsC.expectation_Z(Sigma_A, m_A, sigma_M, Beta, Z_tilde, mu_M, q_Z,
                             neighboursIndexes.astype(np.int32), M, J, K,
                             maxNeighbours)

    def test_expectH(self):
        M = 51
        J = 25
        N = 325
        D = 3
        TR = 1.
        Thrf = 25.
        dt = .5
        data = self.data_simu
        Gamma = np.identity(N)
        Q_barnCond = np.zeros((M, M, D, D), dtype=np.float64)
        XGamma = np.zeros((M, D, N), dtype=np.float64)
        XX = np.zeros((M, N, D), dtype=np.int32)
        Y = data.bold
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        PL = np.dot(P, L)
        y_tilde = Y - np.dot(P, L)
        TT, m_h = getCanoHRF(Thrf, dt)
        m_h = m_h[:D]
        m_H = np.array(m_h)
        sigma_epsilone = np.ones(J)
        Sigma_H = np.ones((D, D), dtype=float)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        scale = 1
        order = 2
        D2 = vt.buildFiniteDiffMatrix(order, D)
        R = np.dot(D2, D2) / pow(dt, 2 * order)
        sigmaH = 0.1
        UtilsC.expectation_H(XGamma, Q_barnCond, sigma_epsilone, Gamma,
                             R, Sigma_H, Y, y_tilde, m_A, m_H, Sigma_A,
                             XX.astype(np.int32), J, D, M, N, scale, sigmaH)

    def test_expectA(self):
        M = 51
        K = 2
        J = 25
        N = 325
        D = 3
        TR = 1.
        Thrf = 25.
        dt = .5
        data = self.data_simu
        Y = data.bold
        Onsets = data.get_joined_onsets()
        Gamma = np.identity(N)
        XX = np.zeros((M, N, D), dtype=np.int32)
        P = vt.PolyMat(N, 4, TR)
        L = vt.polyFit(Y, TR, 4, P)
        PL = np.dot(P, L)
        y_tilde = Y - np.dot(P, L)
        TT, m_h = getCanoHRF(Thrf, dt)
        m_h = m_h[:D]
        sigma_epsilone = np.ones(J)
        m_H = np.array(m_h)
        Sigma_H = np.ones((D, D), dtype=float)
        m_A = np.zeros((J, M), dtype=np.float64)
        Sigma_A = np.zeros((M, M, J), np.float64)
        for j in xrange(0, J):
            Sigma_A[:, :, j] = 0.01 * np.identity(M)
        mu_M = np.zeros((M, K), dtype=np.float64)
        sigma_M = np.ones((M, K), dtype=np.float64)
        q_Z = np.zeros((M, K, J), dtype=np.float64)
        UtilsC.expectation_A(q_Z, mu_M, sigma_M, PL, sigma_epsilone, Gamma,
                             Sigma_H, Y, y_tilde, m_A, m_H, Sigma_A,
                             XX.astype(np.int32), J, D, M, N, K)
